
/*
* Creació de les taules de la BD
* connectat com a admin_tfc/tfc
*/

spool ./03_CreacioTaules.log;


-- MÒDUL D'EXPLOTACIÓ
-- ******************
PROMPT 'CREATE TABLE CLIENTS';
CREATE TABLE CLIENTS (
	IDCLIENT NUMBER(6) NOT NULL ENABLE VALIDATE, 
	NOMCLIENT VARCHAR2(20 char) NOT NULL ENABLE VALIDATE, 
	COGNOMS VARCHAR2(30 char), 
	NIF VARCHAR2(9 char) NOT NULL ENABLE VALIDATE, 
	CORREUELECTRONIC VARCHAR2(25 char), 
	TELEFON VARCHAR2(10 char),
	ESACTIU CHAR DEFAULT 'Y',
	CONSTRAINT CLIENTS_PK PRIMARY KEY(IDCLIENT),
	CONSTRAINT CLIENTS_UK UNIQUE(NIF),
	CONSTRAINT CLIENTS_ECK CHECK (ESACTIU IN ('N','Y'))
   ) 
	TABLESPACE TFC_BD;

PROMPT 'CREATE TABLE MERCATS';	
CREATE TABLE MERCATS (
	IDMERCAT NUMBER(6) NOT NULL ENABLE VALIDATE,
	NOMMERCAT VARCHAR2(20 char) NOT NULL ENABLE VALIDATE,
	CONSTRAINT MERCATS_PK PRIMARY KEY (IDMERCAT),
	CONSTRAINT MERCATS_UK UNIQUE (NOMMERCAT)
	)
	TABLESPACE TFC_BD;

PROMPT 'CREATE TABLE ACCIONS';		
CREATE TABLE ACCIONS (
	IDACCIO NUMBER(6) NOT NULL ENABLE VALIDATE,
	TICKER VARCHAR2(10 char) NOT NULL ENABLE VALIDATE,
	NOMACCIO VARCHAR2(50 char) NOT NULL ENABLE VALIDATE,
	VALORVIGENT NUMBER(6,2),
	MERCAT NUMBER(6),
	ESACTIU CHAR  DEFAULT 'Y',
	CONSTRAINT ACCIONS_PK PRIMARY KEY (IDACCIO),
	CONSTRAINT ACCIONS_UK UNIQUE (TICKER),
	CONSTRAINT ACCIONS_FK1 FOREIGN KEY (MERCAT)
			REFERENCES MERCATS(IDMERCAT),
	CONSTRAINT ACCIONS_ECK CHECK (ESACTIU IN ('N','Y'))
	)
	TABLESPACE TFC_BD;

PROMPT 'CREATE TABLE CARTERA';	
CREATE TABLE CARTERA (
	IDCARTERA NUMBER(10) NOT NULL ENABLE VALIDATE,
	CLIENT NUMBER (6)  NOT NULL ENABLE VALIDATE,
	ACCIO NUMBER (6)  NOT NULL ENABLE VALIDATE,
	QUANTITAT NUMBER (6) NOT NULL ENABLE VALIDATE,
	VALORTOTAL NUMBER(8,2),
	CONSTRAINT CARTERA_PK PRIMARY KEY (IDCARTERA),
	CONSTRAINT CARTERA_UK UNIQUE (CLIENT, ACCIO),
	CONSTRAINT CARTERA_QTAT_CK CHECK (QUANTITAT >= 0),
	CONSTRAINT CARTERA_FK1 FOREIGN KEY (CLIENT)
		REFERENCES CLIENTS(IDCLIENT),
	CONSTRAINT CARTERA_FK2 FOREIGN KEY (ACCIO)
		REFERENCES ACCIONS(IDACCIO)
	)
	TABLESPACE TFC_BD;

PROMPT 'CREATE TABLE HISTORICCARTERA';
CREATE TABLE HISTORICCARTERA (
	IDHISTORICCARTERA NUMBER(10) NOT NULL ENABLE VALIDATE,
	CARTERA NUMBER(10)  NOT NULL ENABLE VALIDATE,
	QUANTITAT NUMBER (6) NOT NULL ENABLE VALIDATE,
	VALORTOTAL NUMBER(8,2),
	DATAINICI timestamp NOT NULL ENABLE VALIDATE, 
	CONSTRAINT HISTORICCARTERA_PK PRIMARY KEY (IDHISTORICCARTERA),
	CONSTRAINT HISTORICCARTERA_UK UNIQUE (CARTERA, DATAINICI),
	CONSTRAINT HISTORICCARTERA_FK1 FOREIGN KEY (CARTERA)
		REFERENCES CARTERA (IDCARTERA)
	)
	TABLESPACE TFC_BD;

PROMPT 'CREATE TABLE HISTORICACCIONS';
CREATE TABLE HISTORICACCIONS (
	IDHISTORICACCIO NUMBER(10) NOT NULL ENABLE VALIDATE,
	ACCIO NUMBER(6)  NOT NULL ENABLE VALIDATE,
	VALORVELL NUMBER (6,2), 
	VALORNOU NUMBER (6,2),
	DATAINICI DATE NOT NULL ENABLE VALIDATE, 
	CONSTRAINT HISTORICACCIONS_PK PRIMARY KEY (IDHISTORICACCIO),
	CONSTRAINT HISTORICACCIONS_UK UNIQUE (ACCIO, DATAINICI),
	CONSTRAINT HISTORICACCIONS_FK1 FOREIGN KEY (ACCIO)
		REFERENCES ACCIONS(IDACCIO)
	)
	TABLESPACE TFC_BD;

PROMPT 'CREATE TABLE ORDRES';
CREATE TABLE ORDRES (
	IDORDRE NUMBER(10) NOT NULL ENABLE VALIDATE,
	CLIENT NUMBER (6) NOT NULL ENABLE VALIDATE,
	ACCIO NUMBER (6) NOT NULL ENABLE VALIDATE,
	DATAENTRADA DATE NOT NULL ENABLE VALIDATE, 
	TIPUS VARCHAR2 (6 CHAR) CHECK (TIPUS IN ('COMPRA', 'VENDA')),
	ORIGEN VARCHAR2 (3 CHAR) CHECK ( ORIGEN IN ('MAN', 'AUT')),
	ESTAT VARCHAR2 (2 CHAR) CHECK ( ESTAT IN ('PD', 'CA', 'EX')),
	PREUOBJECTIU NUMBER(8,2),
	PREUOBTINGUT NUMBER (8,2),
	QUANTITAT NUMBER (8,2) NOT NULL,
	DATAEXECUCIO DATE,
	TOTAL NUMBER (8,2), 
	CONSTRAINT ORDRES_PK PRIMARY KEY (IDORDRE),
	CONSTRAINT ORDRES_FK1 FOREIGN KEY (CLIENT)
		REFERENCES CLIENTS (IDCLIENT),
	CONSTRAINT ORDRES_FK2 FOREIGN KEY (ACCIO)
		REFERENCES ACCIONS (IDACCIO)
	)
	TABLESPACE TFC_BD;

-- MÒDUL ESTADÍSTIC
-- ****************
-- Taules de dimensions

PROMPT 'CREATE TABLE TD_TEMPS';
CREATE TABLE TD_TEMPS (
	IDDATA NUMBER(8) NOT NULL ENABLE VALIDATE,
	DATAANY NUMBER(4),
	MESANY NUMBER(2),
	DIAANY NUMBER(2),
	CONSTRAINT TD_TEMPS_PK PRIMARY KEY (idData)
	)
	TABLESPACE TFC_DWH;

PROMPT 'CREATE TABLE TD_CLIENTS';	
CREATE TABLE TD_CLIENTS (
	IDCLIENT NUMBER (6) NOT NULL ENABLE VALIDATE, 
	NOMCLIENT VARCHAR2(20 char), 
	CONSTRAINT TD_CLIENTS_PK PRIMARY KEY (IDCLIENT)
	)
	TABLESPACE TFC_DWH;

PROMPT 'CREATE TABLE TD_ACCIONS';
CREATE TABLE TD_ACCIONS (
	IDACCIO NUMBER (6) NOT NULL ENABLE VALIDATE, 
	TICKER VARCHAR2(10 char),
	NOMACCIO VARCHAR2(50 char),
	CONSTRAINT TD_ACCIONS_PK PRIMARY KEY (IDACCIO)
	)
	TABLESPACE TFC_DWH;


-- Taules de fets

PROMPT 'CREATE TABLE TF_BENEFICIS';
CREATE TABLE TF_BENEFICIS (
	IDCLIENT NUMBER (6) NOT NULL ENABLE VALIDATE, 
	IDTEMPS NUMBER (8) NOT NULL ENABLE VALIDATE,
	BENEFICINET NUMBER (8,2),
	X100BENEFICI NUMBER (5,2),
	CONSTRAINT TF_BENEFICIS_PK PRIMARY KEY (IDCLIENT, IDTEMPS),
	CONSTRAINT TF_BENEFICIS_FK1 FOREIGN KEY (IDCLIENT)
		REFERENCES TD_CLIENTS(IDCLIENT),
	CONSTRAINT TF_BENEFICIS_FK2 FOREIGN KEY (IDTEMPS)
		REFERENCES TD_TEMPS(IDDATA)
	)
	TABLESPACE TFC_DWH;

PROMPT 'CREATE TABLE TF_CARTERAVALORS';
CREATE TABLE TF_CARTERAVALORS (
	IDTEMPS NUMBER (8) NOT NULL ENABLE VALIDATE,
	IDCLIENT NUMBER (6) NOT NULL ENABLE VALIDATE, 
	NUMACCIONS NUMBER (6),
	VALORCARTERA NUMBER (8,2),
	CONSTRAINT TF_CARTERAVALORS_PK PRIMARY KEY (IDCLIENT, IDTEMPS),
	CONSTRAINT TF_CARTERAVALORS_FK1 FOREIGN KEY (IDCLIENT)
		REFERENCES TD_CLIENTS(IDCLIENT),
	CONSTRAINT TF_CARTERAVALORS_FK2 FOREIGN KEY (IDTEMPS)
		REFERENCES TD_TEMPS(IDDATA)
	)
	TABLESPACE TFC_DWH;

PROMPT 'CREATE TABLE TF_ACCIOMESNEGOCIADA';	
CREATE TABLE TF_ACCIOMESNEGOCIADA (
	IDTEMPS NUMBER (8) NOT NULL ENABLE VALIDATE,
	IDACCIO NUMBER (6) NOT NULL ENABLE VALIDATE, 
	NUMOPERACIONS NUMBER (6),
	CONSTRAINT TF_ACCIOMESNEGOCIADA_PK PRIMARY KEY (IDACCIO, IDTEMPS),
	CONSTRAINT TF_ACCIOMESNEGOCIADA_FK1 FOREIGN KEY (IDACCIO)
		REFERENCES TD_ACCIONS(IDACCIO),
	CONSTRAINT TF_ACCIOMESNEGOCIADA_FK2 FOREIGN KEY (IDTEMPS)
		REFERENCES TD_TEMPS(IDDATA)
	)
	TABLESPACE TFC_DWH;

PROMPT 'CREATE TABLE TF_ACCIONSDISPONIBLES';
CREATE TABLE TF_ACCIONSDISPONIBLES (
	IDTEMPS NUMBER (8) NOT NULL ENABLE VALIDATE,
	IDACCIO NUMBER (6) NOT NULL ENABLE VALIDATE, 
	NUMACCIONSENVENDA NUMBER (6),
	NUMCLIENTS NUMBER (6),
	CONSTRAINT TF_ACCIONSDISPONIBLES_PK PRIMARY KEY (IDACCIO, IDTEMPS),
	CONSTRAINT TF_ACCIONSDISPONIBLES_FK1 FOREIGN KEY (IDACCIO)
		REFERENCES TD_ACCIONS(IDACCIO),
	CONSTRAINT TF_ACCIONSDISPONIBLES_FK2 FOREIGN KEY (IDTEMPS)
		REFERENCES TD_TEMPS(IDDATA)
	)
	TABLESPACE TFC_DWH;

-- MÒDUL DE CONTROL
-- ****************

PROMPT 'CREATE TABLE LOG_PROCEDIMENTS';
CREATE TABLE LOG_PROCEDIMENTS (
	idLog numeric(10) not null,
	NomProcediment Varchar2(50 CHAR) not null,
	tempsExecucio timestamp, 
    parametresIn Varchar2(100 CHAR),
    finalitzacio Varchar2(10 CHAR),
    finalitzacioTxt varchar2(200 CHAR) not null,
     CONSTRAINT lpe_pk  PRIMARY KEY (idLog)
	)
	TABLESPACE TFC_BD;

PROMPT 'CREATE TABLE ERRORS_RECONEGUTS';
CREATE TABLE ERRORS_RECONEGUTS (
	idError numeric(3) not null,
	descripcio Varchar2(100 CHAR) not null,
	   CONSTRAINT ERRORS_RECONEGUTS_PK PRIMARY KEY (idError)
	)
	TABLESPACE TFC_BD;

spool off;
