/* 
* Paquet accions
* conté els mètodes: Alta, modificació, baixa lògica
*
*/

spool ./pkg_accions.log;
--capçalera paquet
create or replace package pkg_accions is

--procediment per donar d'alta una acció 
procedure prc_accio_alta (
	pTicker VARCHAR2, 
	pNomAccio VARCHAR2, 
	pMercat NUMBER,
	rsp OUT varchar2
	);
	
--procediment per modificar les dades principals d'una acció, donat el seu id
procedure prc_accio_modificacio (
	pIdAccio NUMBER,
	pTicker VARCHAR2, 
	pNomAccio VARCHAR2, 
	pMercat NUMBER,
	rsp OUT varchar2
	);

--procediment per donar de baixa lògica una acció amb id passat per paràmetre
procedure prc_accio_baixa (
	pIdAccio NUMBER,
	rsp OUT varchar2
	);

--procediment per donar de baixa lògica una acció amb el Ticker passat per paràmetre. Sobrecàrrega.
procedure prc_accio_baixa (
	pTicker VARCHAR2,
	rsp OUT varchar2
	);

-- Procediment per actualitzar el preu de l'acció amb el ID passat per paràmetre	
procedure prc_accio_preu (
	pIdAccio NUMBER,
	pPreu NUMBER,
	rsp OUT varchar2
	);

-- Procediment per actualitzar el preu de l'acció amb el TICKER passat per paràmetre	
procedure prc_accio_preu (
	pTicker VARCHAR2, 
	pPreu NUMBER,
	rsp OUT varchar2
	);

end pkg_accions;
/

-- cos del paquet
CREATE OR REPLACE PACKAGE BODY pkg_accions is

/*
* definicio del procediment d'alta d'accions
*
*/
procedure prc_accio_alta (
	pTicker VARCHAR2, 
	pNomAccio VARCHAR2, 
	pMercat NUMBER,
	rsp OUT varchar2
	)  IS
-- declaració de variables locals del procediment
	prc_nom varchar2(25 CHAR):='PRC_ACCIO_ALTA';
	prc_paramIN varchar(100 CHAR) := null;
	sError varchar(512);
BEGIN

-->>recull dels paràmetres d'entrada a un string
	prc_paramIN := pTicker||', '||pNomAccio||', '||pMercat;
--<<fi del recull dels paràmetres d'entrada
	
--inserció a la taula ACCIONS del nou registre amb les dades passades
	INSERT INTO accions (idaccio, ticker, nomaccio, mercat)
		VALUES (accions_seq.nextval, UPPER(pTicker), UPPER(pNomAccio), pMercat);

--registra a la taula Log l'execució del procediment
	pkg_control.prc_logProcediment(prc_nom, prc_paramIN, 'FI OK');
--s'informa del paràmetre de sortida
	rsp := 'OK';

	EXCEPTION
		WHEN DUP_VAL_ON_INDEX THEN
			sError := pkg_control.fun_error(3)||substr(SQLERRM, 1, 500);
--registra a la taula Log l'execució del procediment
			pkg_control.prc_logProcediment(prc_nom, prc_paramIN, sError);
-- s'informa del paràmetre de sortida
			rsp := sError;

		WHEN others THEN
			sError := pkg_control.fun_error(1)||substr(SQLERRM, 1, 500);
--registra a la taula Log l'execució del procediment
			pkg_control.prc_logProcediment(prc_nom, prc_paramIN, sError);
-- s'informa del paràmetre de sortida
		rsp := sError;

END prc_accio_alta;
	
/*
* procediment per modificar les dades principals d'una accio, donat el seu id
*
*/

procedure prc_accio_modificacio (
	pIdAccio NUMBER,
	pTicker VARCHAR2, 
	pNomAccio VARCHAR2, 
	pMercat NUMBER,
	rsp OUT varchar2
	) is

-- declaració de variables locals del procediment
	prc_nom varchar2(25 CHAR):='PRC_ACCIO_MODIFICACIO';
	prc_paramIN varchar(100 CHAR) := null;
	sError varchar(512);
	var_accio_local accions%rowtype;
	situacioNovalida EXCEPTION;

BEGIN
-->>recull dels paràmetres d'entrada a un string
	prc_paramIN := pIdAccio||', '||pTicker||', '||pNomAccio||', '||pMercat;
--<<fi del recull dels paràmetres d'entrada

	if pIdAccio is null THEN
		raise admin_tfc.pkg_control.insert_null;
	end if;

--es sel.lecciona l'acció amb el id passat per paràmetre
-- i es modifica el registre  de la taula accions amb les noves dades passades
  SELECT * 
	INTO var_accio_local
	from accions ac
	where ac.idaccio = pIdAccio;

	if var_accio_local.esactiu != 'Y' THEN
		raise situacioNovalida;
	end if;

--si el paràmetre d'entrada no és null s'actualitza el registre, si és null es manté el valor anterior
	var_accio_local.ticker := NVL(pTicker, var_accio_local.ticker);
	var_accio_local.nomaccio := NVL (pNomAccio, var_accio_local.nomaccio);
	var_accio_local.mercat := NVL (pMercat, var_accio_local.mercat);
	
	--DBMS_OUTPUT.PUT_LINE ('fitxer trobat '|| pIdAccio||' '||pTicker);
	
--actualització de la taula accions amb els nous valors, només si és activa
	UPDATE accions 
		set row = var_accio_local 
		where idaccio = pIdAccio and esActiu = 'Y';
	

--registra a la taula Log l'execució del procediment
	pkg_control.prc_logProcediment(prc_nom, prc_paramIN, 'FI OK');
--s'informa del paràmetre de sortida
	rsp := 'OK';

 	EXCEPTION
		WHEN situacioNovalida THEN
			sError :=  pkg_control.fun_error(4)||substr(SQLERRM, 1, 500);
--registra a la taula Log l'execució del procediment
			pkg_control.prc_logProcediment(prc_nom, prc_paramIN, sError);
-- s'informa del paràmetre de sortida
			rsp := sError;

		WHEN NO_DATA_FOUND THEN
			sError :=  pkg_control.fun_error(2)||substr(SQLERRM, 1, 500);
--registra a la taula Log l'execució del procediment
			pkg_control.prc_logProcediment(prc_nom, prc_paramIN, sError);
-- s'informa del paràmetre de sortida
			rsp := sError;

		WHEN DUP_VAL_ON_INDEX THEN
			sError :=  pkg_control.fun_error(3)||substr(SQLERRM, 1, 500);
--registra a la taula Log l'execució del procediment
			pkg_control.prc_logProcediment(prc_nom, prc_paramIN, sError);
-- s'informa del paràmetre de sortida
			rsp := sError;

		WHEN others THEN
			sError := pkg_control.fun_error(1)||substr(SQLERRM, 1, 500);
--registra a la taula Log l'execució del procediment
			pkg_control.prc_logProcediment(prc_nom, prc_paramIN, sError);
-- s'informa del paràmetre de sortida
			rsp := sError;

end prc_accio_modificacio;

/*
* procediment per donar de baixa lògica una acció amb id passat per paràmetre
* es sobrecarregarà amb un altre paquet per ticker
* 
*/
procedure prc_accio_baixa (
	pIdAccio NUMBER,
	rsp OUT varchar2
	) is

-- declaració de variables locals del procediment
	prc_nom varchar2(25 CHAR):='PRC_ACCIO_BAIXA';
	prc_paramIN varchar(100 CHAR) := null;
	sError varchar(512);
	comptador number(4);
BEGIN
-->>recull dels paràmetres d'entrada a un string
	prc_paramIN := pIdAccio;
--<<fi del recull dels paràmetres d'entrada
	
	if pIdAccio is null THEN
		raise admin_tfc.pkg_control.insert_null;
	end if;

	UPDATE ORDRES O
	SET estat ='CA'
	WHERE O.ACCIO = pIdAccio;
	

	UPDATE ACCIONS AC
	SET esactiu = 'N'
	WHERE AC.IDACCIO = pIdAccio;
	comptador := SQL%ROWCOUNT;
	

	IF comptador >= 1 THEN
	--registra a la taula Log l'execució del procediment
		pkg_control.prc_logProcediment(prc_nom, prc_paramIN, 'FI OK');
	--s'informa del paràmetre de sortida
		rsp := 'OK';
	ELSE
		RAISE NO_DATA_FOUND;

	END IF;

	EXCEPTION
		WHEN NO_DATA_FOUND THEN
			sError := pkg_control.fun_error(4)||substr(SQLERRM, 1, 500);
--registra a la taula Log l'execució del procediment
			pkg_control.prc_logProcediment(prc_nom, prc_paramIN, sError);
-- s'informa del paràmetre de sortida
			rsp := sError;

		WHEN others THEN
			sError := pkg_control.fun_error(1)||substr(SQLERRM, 1, 500);
--registra a la taula Log l'execució del procediment
			pkg_control.prc_logProcediment(prc_nom, prc_paramIN, sError);
-- s'informa del paràmetre de sortida
			rsp := sError;



END prc_accio_baixa;

/*
* procediment per donar de baixa lògica una acció amb el TICKER passat per paràmetre
* es sobrecarregarà amb un altre paquet per id
* 
*/
procedure prc_accio_baixa (
	pTicker VARCHAR2, 
	rsp OUT varchar2
	) is

-- declaració de variables locals del procediment
	prc_nom varchar2(25 CHAR):='PRC_ACCIO_BAIXA';
	prc_paramIN varchar(100 CHAR) := null;
	sError varchar(512);
	capRegistre EXCEPTION; -- es defineix una excepció pròpia
	comptador number(4);
BEGIN
-->>recull dels paràmetres d'entrada a un string
	prc_paramIN := pTicker;
--<<fi del recull dels paràmetres d'entrada
	
	if pTicker is null THEN
		raise admin_tfc.pkg_control.insert_null;
	end if;

	UPDATE ORDRES O
	SET estat ='CA'
	WHERE O.ACCIO = (select idAccio from ACCIONS A
			where A.TICKER = pTicker);
	
	UPDATE ACCIONS AC
	SET esactiu = 'N'
	WHERE AC.IDACCIO = (select idAccio from ACCIONS A
			where A.TICKER = pTicker);
	comptador := SQL%ROWCOUNT;
	

IF comptador >= 1 THEN
	--registra a la taula Log l'execució del procediment
		pkg_control.prc_logProcediment(prc_nom, prc_paramIN, 'FI OK');
	--s'informa del paràmetre de sortida
		rsp := 'OK';
	ELSE
		RAISE NO_DATA_FOUND;

	END IF;

	EXCEPTION
		WHEN NO_DATA_FOUND THEN
			sError := pkg_control.fun_error(4)||substr(SQLERRM, 1, 500);
--registra a la taula Log l'execució del procediment
			pkg_control.prc_logProcediment(prc_nom, prc_paramIN, sError);
-- s'informa del paràmetre de sortida
		rsp := sError;

		WHEN others THEN
			sError := pkg_control.fun_error(1)||substr(SQLERRM, 1, 500);
--registra a la taula Log l'execució del procediment
			pkg_control.prc_logProcediment(prc_nom, prc_paramIN, sError);
-- s'informa del paràmetre de sortida
			rsp := sError;

END prc_accio_baixa;

/*
* Actualització del preu de l'acció passant el id per paràmetre, sempre la acció estigui activa
* es sobrecarregarà amb el mètode que es passa el Ticker
*/
procedure prc_accio_preu (
	pIdAccio NUMBER,
	pPreu NUMBER,
	rsp OUT varchar2
	) is

-- declaració de variables locals del procediment
	prc_nom varchar2(25 CHAR):='PRC_ACCIO_PREU';
	prc_paramIN varchar(100 CHAR) := null;
	sError varchar(512);
	capRegistre EXCEPTION; -- es defineix una excepció pròpia
	comptador number(4);
BEGIN
-->>recull dels paràmetres d'entrada a un string
	prc_paramIN := pIdAccio||', '||pPreu;
--<<fi del recull dels paràmetres d'entrada
	
	if pIdAccio is null THEN
		raise admin_tfc.pkg_control.insert_null;
	end if;

	UPDATE ACCIONS AC
	SET VALORVIGENT = pPreu
	WHERE AC.IDACCIO = pIdAccio
		and AC.ESACTIU = 'Y';
	comptador := SQL%ROWCOUNT;

-- s'executen les ordres que cumpleixen la condició de preu objectiu
	pkg_ordre.prc_ordre_execucio_auto (pIdAccio, pPreu, sError);

-- s'actualitza la taula cartera amb el nou preu de l'acció
	pkg_cartera.prc_cartera_actualitzar(pIdAccio, pPreu, sError);



IF comptador >= 1 THEN
	--registra a la taula Log l'execució del procediment
		pkg_control.prc_logProcediment(prc_nom, prc_paramIN, 'FI OK');
	--s'informa del paràmetre de sortida
		rsp := 'OK';
	ELSE
		RAISE capRegistre;

	END IF;

-- s'actualitza la taula TF_carteraValors del mòdul estadístic
	pkg_dwh.prc_ins_tfCarteraValors;

-- s'actualitza la taula TF_beneficis del mòdul estadístic
	pkg_dwh.prc_ins_tfBeneficis;

	EXCEPTION
		WHEN capRegistre THEN
			sError := pkg_control.fun_error(4)||substr(SQLERRM, 1, 500);
--registra a la taula Log l'execució del procediment
			pkg_control.prc_logProcediment(prc_nom, prc_paramIN, sError);
-- s'informa del paràmetre de sortida
			rsp := sError;

		WHEN others THEN
			sError := pkg_control.fun_error(1)||substr(SQLERRM, 1, 500);
--registra a la taula Log l'execució del procediment
			pkg_control.prc_logProcediment(prc_nom, prc_paramIN, sError);
-- s'informa del paràmetre de sortida
			rsp := sError;

END prc_accio_preu;

/*
* Actualització del preu de l'acció passant el TICKER per paràmetre, sempre la acció estigui activa
* es sobrecarregarà amb el mètode que es passa el ID
*/
procedure prc_accio_preu (
	pTicker VARCHAR2, 
	pPreu NUMBER,
	rsp OUT varchar2
	) is

-- declaració de variables locals del procediment
	prc_nom varchar2(25 CHAR):='PRC_ACCIO_PREU';
	prc_paramIN varchar(100 CHAR) := null;
	sError varchar(512);
	capRegistre EXCEPTION; -- es defineix una excepció pròpia
	comptador number(4);
	var_idaccio accions.idAccio%type;
BEGIN
-->>recull dels paràmetres d'entrada a un string
	prc_paramIN := pTicker||', '||pPreu;
--<<fi del recull dels paràmetres d'entrada
	
	if pTicker is null THEN
		raise admin_tfc.pkg_control.insert_null;
	end if;
	
-- obtemnim el id de l'acció 
	select idAccio 
	into var_idaccio
	from ACCIONS A
	where A.TICKER = pTicker;

	UPDATE ACCIONS AC
	SET VALORVIGENT = pPreu
	WHERE AC.IDACCIO = var_idaccio
		and AC.ESACTIU = 'Y';
	comptador := SQL%ROWCOUNT;
	
-- s'executen les ordres que cumpleixen la condició de preu objectiu
	pkg_ordre.prc_ordre_execucio_auto (var_idaccio, pPreu, sError);

-- s'actualitza la taula cartera amb el nou preu de l'acció
	pkg_cartera.prc_cartera_actualitzar(var_idaccio, pPreu, sError);




IF comptador >= 1 THEN
	--registra a la taula Log l'execució del procediment
		pkg_control.prc_logProcediment(prc_nom, prc_paramIN, 'FI OK');
	--s'informa del paràmetre de sortida
		rsp := 'OK';
	ELSE
		RAISE capRegistre;

	END IF;

-- s'actualitza la taula TF_carteraValors del mòdul estadístic
	pkg_dwh.prc_ins_tfCarteraValors;

-- s'actualitza la taula TF_beneficis del mòdul estadístic
	pkg_dwh.prc_ins_tfBeneficis;

	EXCEPTION
		WHEN capRegistre THEN
			sError := pkg_control.fun_error(4)||substr(SQLERRM, 1, 500);
--registra a la taula Log l'execució del procediment
			pkg_control.prc_logProcediment(prc_nom, prc_paramIN, sError);
-- s'informa del paràmetre de sortida
			rsp := sError;
		WHEN NO_DATA_FOUND THEN
			sError := pkg_control.fun_error(2)||substr(SQLERRM, 1, 500);
--registra a la taula Log l'execució del procediment
			pkg_control.prc_logProcediment(prc_nom, prc_paramIN, sError);
-- s'informa del paràmetre de sortida
			rsp := sError;

		WHEN others THEN
			sError := pkg_control.fun_error(1)||substr(SQLERRM, 1, 500);
--registra a la taula Log l'execució del procediment
			pkg_control.prc_logProcediment(prc_nom, prc_paramIN, sError);
-- s'informa del paràmetre de sortida
			rsp := sError;

END prc_accio_preu;





-- fi del paquet
end pkg_accions;
/

spool off;