/* 
* Paquet cartera
* conté els mètodes: adjudicacio d'accions, retirada d'accions, 
*
*/

spool ./pkg_cartera.log;
--capçalera paquet
create or replace package pkg_cartera is

--procediment per adjudicar accions a un client
procedure prc_cartera_adjudicar (
	pIdClient NUMBER, 
	pIdAccio NUMBER,
	pQuantitat NUMBER, 
	rsp OUT varchar2
	);

--procediment per adjudicar accions a un client
procedure prc_cartera_adjudicar (
	pIdClient NUMBER, 
	pIdAccio NUMBER,
	pQuantitat NUMBER, 
	pValorVigent NUMBER,
	rsp OUT varchar2
	);

-- procediment per retirar accions a un client
procedure prc_cartera_retirar (
	pIdClient NUMBER, 
	pIdAccio NUMBER,
	pQuantitat NUMBER, 
	rsp OUT varchar2
	);

-- procediment per retirar accions a un client
procedure prc_cartera_retirar (
	pIdClient NUMBER, 
	pIdAccio NUMBER,
	pQuantitat NUMBER,
	pValorVigent NUMBER, 
	rsp OUT varchar2
	);

--procediment per actualitzar el valor de la cartera d'acord a la variació de preu de l'acció
-- procediment manual que es passa l'accio i el seu preu i s'actualitzarà el valor de totes les carteres que la contenen
procedure prc_cartera_actualitzar (
	pIdAccio NUMBER,
	pPreuVigent NUMBER,
	rsp OUT varchar2
	);

--procediment per actualitzar totes les carteres d'acord a la variació de preu de l'acció
-- procediment automatic que actualitza tots els registres
-- es sobrecarrega el procediment
procedure prc_cartera_actualitzar (
	rsp OUT varchar2
	);


end pkg_cartera;
/

-- cos del paquet
CREATE OR REPLACE PACKAGE BODY pkg_cartera is

/*
* definicio del procediment d'adjudicació d'accions a un client
* es passa els paràmetres de l'identificador del client, de l'acció i la quantitat a adjudicar
*
*/
procedure prc_cartera_adjudicar (
	pIdClient NUMBER, 
	pIdAccio NUMBER,
	pQuantitat NUMBER, 
	rsp OUT varchar2
	) IS
-- declaració de variables locals del procediment
	prc_nom varchar2(25 CHAR):='PRC_CARTERA_ADJUDICAR';
	prc_paramIN varchar(100 CHAR) := null;
	sError varchar(512);
	var_preu accions.valorvigent%type;
	var_client clients.nomclient%type;
	numAccions cartera.quantitat%type;
BEGIN

-->>recull dels paràmetres d'entrada a un string
	prc_paramIN := pIdClient||', '||pIdAccio||', '||pQuantitat;
--<<fi del recull dels paràmetres d'entrada

	
	if pIdClient is null or pIdAccio is null or pQuantitat is null then
		raise admin_tfc.pkg_control.insert_null;
	end if;

-- obtenim el valor vigent de l'acció
	select valorvigent 
	into var_preu
	from accions
	where idaccio = pIdAccio;

-- verifiquem si existeix el client
	select nomClient
	into var_client
	from clients
	where idclient = pIdClient;

--inserció a la taula CARTERA del nou registre amb les dades passades
	INSERT INTO cartera (idcartera, client, accio, quantitat, valorTotal)
		VALUES (cartera_seq.nextval, pIdClient, pIdAccio, pQuantitat, pQuantitat*var_preu);

--registra a la taula Log l'execució del procediment
	pkg_control.prc_logProcediment(prc_nom, prc_paramIN, 'FI OK');
--s'informa del paràmetre de sortida
	rsp := 'OK';

	EXCEPTION
		WHEN DUP_VAL_ON_INDEX THEN --capturem l'excepció d'unicitat que indica que aquella cartera ja existeix,
--per tant, s'ha de modificar el registra ja existent, enlloc de insertar-ne un de nou
		
-- obtenim la quantitat d'accions que hi han en la cartera en aquell moment
		select quantitat
		into numAccions
		from cartera
		WHERE client = pIdClient and accio = pIdAccio;


		UPDATE cartera 
			SET quantitat = numAccions + pQuantitat, valorTotal = (numAccions + pQuantitat)*var_preu
			WHERE client = pIdClient and accio = pIdAccio;

--registra a la taula Log l'execució del procediment
		pkg_control.prc_logProcediment(prc_nom, prc_paramIN, 'FI OK');
--s'informa del paràmetre de sortida
		rsp := 'OK';

		WHEN admin_tfc.pkg_control.parent_key_nf THEN
			sError := pkg_control.fun_error(4)||substr(SQLERRM, 1, 500);
--registra a la taula Log l'execució del procediment
			pkg_control.prc_logProcediment(prc_nom, prc_paramIN, sError);
-- s'informa del paràmetre de sortida
			rsp := sError;

		WHEN others THEN
			sError := pkg_control.fun_error(1)||substr(SQLERRM, 1, 500);
--registra a la taula Log l'execució del procediment
			pkg_control.prc_logProcediment(prc_nom, prc_paramIN, sError);
-- s'informa del paràmetre de sortida
			rsp := sError;

END prc_cartera_adjudicar;


/*
* definicio del procediment d'adjudicació d'accions a un client
* es passa els paràmetres de l'identificador del client, de l'acció, la quantitat a adjudicar i el nou preu
*
*/
procedure prc_cartera_adjudicar (
	pIdClient NUMBER, 
	pIdAccio NUMBER,
	pQuantitat NUMBER,
	pValorVigent NUMBER, 
	rsp OUT varchar2
	) IS
-- declaració de variables locals del procediment
	prc_nom varchar2(25 CHAR):='PRC_CARTERA_ADJUDICAR';
	prc_paramIN varchar(100 CHAR) := null;
	sError varchar(512);
	numAccions cartera.quantitat%type;
BEGIN

-->>recull dels paràmetres d'entrada a un string
	prc_paramIN := pIdClient||', '||pIdAccio||', '||pQuantitat||', '||pValorVigent;
--<<fi del recull dels paràmetres d'entrada
	
	if pIdClient is null or pIdAccio is null or pQuantitat is null or pValorVigent is null then
		raise admin_tfc.pkg_control.insert_null;
	end if;


--inserció a la taula CARTERA del nou registre amb les dades passades
	INSERT INTO cartera (idcartera, client, accio, quantitat, valorTotal)
		VALUES (cartera_seq.nextval, pIdClient, pIdAccio, pQuantitat, pQuantitat*pValorVigent);

--registra a la taula Log l'execució del procediment
	pkg_control.prc_logProcediment(prc_nom, prc_paramIN, 'FI OK');
--s'informa del paràmetre de sortida
	rsp := 'OK';

	EXCEPTION
		WHEN DUP_VAL_ON_INDEX THEN --capturem l'excepció d'unicitat que indica que aquella cartera ja existeix,
--per tant, s'ha de modificar el registra ja existent, enlloc de insertar-ne un de nou
		
-- obtenim la quantitat d'accions que hi han en la cartera en aquell moment
		select quantitat
		into numAccions
		from cartera
		WHERE client = pIdClient and accio = pIdAccio;


		UPDATE cartera 
			SET quantitat = numAccions + pQuantitat, valorTotal = (numAccions + pQuantitat)*pValorVigent
			WHERE client = pIdClient and accio = pIdAccio;

--registra a la taula Log l'execució del procediment
		pkg_control.prc_logProcediment(prc_nom, prc_paramIN, 'FI OK');
--s'informa del paràmetre de sortida
		rsp := 'OK';

		WHEN others THEN
			sError := pkg_control.fun_error(1)||substr(SQLERRM, 1, 500);
--registra a la taula Log l'execució del procediment
			pkg_control.prc_logProcediment(prc_nom, prc_paramIN, sError);
-- s'informa del paràmetre de sortida
			rsp := sError;

END prc_cartera_adjudicar;


/*
* definicio del procediment de retirada d'accions a un client
* es passa els paràmetres de l'identificador del client, de l'acció i la quantitat a retirar
* tindrà en compte si el client té prous accions per poder fer efectiva la retirada
*
*/
procedure prc_cartera_retirar (
	pIdClient NUMBER, 
	pIdAccio NUMBER,
	pQuantitat NUMBER, 
	rsp OUT varchar2
	) IS
-- declaració de variables locals del procediment
	prc_nom varchar2(25 CHAR):='PRC_CARTERA_RETIRAR';
	prc_paramIN varchar(100 CHAR) := null;
	sError varchar(512);
	numAccions cartera.quantitat%type;
	comptador number(4) := 0;
	var_preu accions.valorvigent%type;
	poquesAccions EXCEPTION; -- es defineix una excepció pròpia
	capRegistre EXCEPTION;

BEGIN

-->>recull dels paràmetres d'entrada a un string
	prc_paramIN := pIdClient||', '||pIdAccio||', '||pQuantitat;
--<<fi del recull dels paràmetres d'entrada
	
	if pIdClient is null or pIdAccio is null or pQuantitat is null then
		raise admin_tfc.pkg_control.insert_null;
	end if;

-- obtenim el valor vigent de l'acció
	select valorvigent 
	into var_preu
	from accions
	where idaccio = pIdAccio;

-- es fa comprovació de quantes accions té el client en cartera
	select quantitat
	into numAccions
	from cartera
	WHERE client = pIdClient and accio = pIdAccio;

	IF numAccions >= pQuantitat THEN
		UPDATE cartera 
			SET quantitat = numAccions - pQuantitat, valorTotal = (numAccions - pQuantitat)*var_preu
			WHERE client = pIdClient and accio = pIdAccio;
		comptador := SQL%ROWCOUNT;
		
	ELSE 
		
		RAISE poquesAccions;

	END IF;

	IF comptador >= 1 THEN
--registra a la taula Log l'execució del procediment
		pkg_control.prc_logProcediment(prc_nom, prc_paramIN, 'FI OK');
--s'informa del paràmetre de sortida
		rsp := 'OK';
	
	ELSE
		RAISE capRegistre;

	END IF;

	EXCEPTION
		WHEN poquesAccions THEN
			sError := pkg_control.fun_error(5)||substr(SQLERRM, 1, 500);
--registra a la taula Log l'execució del procediment
			pkg_control.prc_logProcediment(prc_nom, prc_paramIN, sError);
-- s'informa del paràmetre de sortida
			rsp := sError;
		WHEN NO_DATA_FOUND THEN 
			sError := pkg_control.fun_error(2)||substr(SQLERRM, 1, 500);
--registra a la taula Log l'execució del procediment
			pkg_control.prc_logProcediment(prc_nom, prc_paramIN, sError);
-- s'informa del paràmetre de sortida
			rsp := sError;
		WHEN capRegistre THEN
			sError := pkg_control.fun_error(4)||substr(SQLERRM, 1, 500);
--registra a la taula Log l'execució del procediment
			pkg_control.prc_logProcediment(prc_nom, prc_paramIN, sError);
-- s'informa del paràmetre de sortida
			rsp := sError;	
		WHEN others THEN
			sError := pkg_control.fun_error(1)||substr(SQLERRM, 1, 500);
--registra a la taula Log l'execució del procediment
			pkg_control.prc_logProcediment(prc_nom, prc_paramIN, sError);
-- s'informa del paràmetre de sortida
			rsp := sError;
		

END prc_cartera_retirar;

/*
* definicio del procediment de retirada d'accions a un client
* es passa els paràmetres de l'identificador del client, de l'acció, la quantitat a retirar i el valor vigent
* tindrà en compte si el client té prous accions per poder fer efectiva la retirada
*
*/
procedure prc_cartera_retirar (
	pIdClient NUMBER, 
	pIdAccio NUMBER,
	pQuantitat NUMBER, 
	pValorVigent NUMBER,
	rsp OUT varchar2
	) IS
-- declaració de variables locals del procediment
	prc_nom varchar2(25 CHAR):='PRC_CARTERA_RETIRAR';
	prc_paramIN varchar(100 CHAR) := null;
	sError varchar(512);
	numAccions cartera.quantitat%type;
	comptador number(4) := 0;
	poquesAccions EXCEPTION; -- es defineix una excepció pròpia
	capRegistre EXCEPTION;

BEGIN

-->>recull dels paràmetres d'entrada a un string
	prc_paramIN := pIdClient||', '||pIdAccio||', '||pQuantitat||', '||pValorVigent;
--<<fi del recull dels paràmetres d'entrada

	if pIdClient is null or pIdAccio is null or pQuantitat is null or pValorVigent is null then
		raise admin_tfc.pkg_control.insert_null;
	end if;

-- es fa comprovació de quantes accions té el client en cartera
	select quantitat
	into numAccions
	from cartera
	WHERE client = pIdClient and accio = pIdAccio;

	IF numAccions >= pQuantitat THEN
		UPDATE cartera 
			SET quantitat = numAccions - pQuantitat, valorTotal = (numAccions - pQuantitat)*pValorVigent
			WHERE client = pIdClient and accio = pIdAccio;
		comptador := SQL%ROWCOUNT;
		
	ELSE 
		
		RAISE poquesAccions;

	END IF;

	IF comptador >= 1 THEN
--registra a la taula Log l'execució del procediment
		pkg_control.prc_logProcediment(prc_nom, prc_paramIN, 'FI OK');
--s'informa del paràmetre de sortida
		rsp := 'OK';
	
	ELSE
		RAISE capRegistre;

	END IF;

	EXCEPTION
		WHEN poquesAccions THEN
			sError := pkg_control.fun_error(5)||substr(SQLERRM, 1, 500);
--registra a la taula Log l'execució del procediment
			pkg_control.prc_logProcediment(prc_nom, prc_paramIN, sError);
-- s'informa del paràmetre de sortida
			rsp := sError;
		WHEN NO_DATA_FOUND THEN 
			sError := pkg_control.fun_error(2)||substr(SQLERRM, 1, 500);
--registra a la taula Log l'execució del procediment
			pkg_control.prc_logProcediment(prc_nom, prc_paramIN, sError);
-- s'informa del paràmetre de sortida
			rsp := sError;
		WHEN capRegistre THEN
			sError := pkg_control.fun_error(4)||substr(SQLERRM, 1, 500);
--registra a la taula Log l'execució del procediment
			pkg_control.prc_logProcediment(prc_nom, prc_paramIN, sError);
-- s'informa del paràmetre de sortida
			rsp := sError;	
		WHEN others THEN
			sError := pkg_control.fun_error(1)||substr(SQLERRM, 1, 500);
--registra a la taula Log l'execució del procediment
			pkg_control.prc_logProcediment(prc_nom, prc_paramIN, sError);
-- s'informa del paràmetre de sortida
			rsp := sError;
		

END prc_cartera_retirar;


/*
* definicio del procediment d'actualització d'una cartera d'accions
* es passa l'identificador de l'acció
* actualizarà el valor de les carteres associades a l'acció passada, a al preu vigent de l'acció.
* només s'actualitzarà les carteres actives, que śon les que estan associades a 
* clients i accions actius
*
*/
procedure prc_cartera_actualitzar (
	pIdAccio NUMBER,
	pPreuVigent NUMBER,
	rsp OUT varchar2
	) IS

 -- declaració de variables locals del procediment
	prc_nom varchar2(25 CHAR):='PRC_CARTERA_ACTUALITZAR';
	prc_paramIN varchar(100 CHAR) := null;
	sError varchar(512);
	numAccions cartera.quantitat%type;
	var_preu accions.valorvigent%type;

	cursor cursorCartera
	IS
		select *
			from cartera ca 
				join clients cl
				  on cl.idclient = ca.client
				join accions a
					on a.idaccio = ca.accio
			where ca.accio = pIdAccio 
				and cl.esactiu = 'Y'
				and a.esactiu = 'Y'
			for update of valorTotal;

	cartera_reg cursorCartera%rowtype;
	
BEGIN

-->>recull dels paràmetres d'entrada a un string
	prc_paramIN := pIdAccio||', '||pPreuVigent;
--<<fi del recull dels paràmetres d'entrada
	
	if pIdAccio is null or pPreuVigent is null then
		raise admin_tfc.pkg_control.insert_null;
	end if;


-- actualitzem el valor de la cartera
	open cursorCartera;
	fetch cursorCartera into cartera_reg;
	if cursorCartera%notfound THEN
		raise NO_DATA_FOUND;
	end if;
	while cursorCartera%found loop 
		UPDATE cartera 
			SET valorTotal = cartera_reg.quantitat*pPreuVigent
			WHERE CURRENT OF cursorCartera;
	fetch cursorCartera into cartera_reg;	
	END LOOP;	
--registra a la taula Log l'execució del procediment
	pkg_control.prc_logProcediment(prc_nom, prc_paramIN, 'FI OK');
--s'informa del paràmetre de sortida
	rsp := 'OK';

	EXCEPTION
		WHEN NO_DATA_FOUND THEN
			sError := pkg_control.fun_error(2)||substr(SQLERRM, 1, 500);
--registra a la taula Log l'execució del procediment
			pkg_control.prc_logProcediment(prc_nom, prc_paramIN, sError);
-- s'informa del paràmetre de sortida
			rsp := sError;

		WHEN others THEN
			sError := pkg_control.fun_error(1)||substr(SQLERRM, 1, 500);
--registra a la taula Log l'execució del procediment
			pkg_control.prc_logProcediment(prc_nom, prc_paramIN, sError);
-- s'informa del paràmetre de sortida
			rsp := sError;


END prc_cartera_actualitzar;


/*
* definicio del procediment AUTOMÀTIC d'actualització d'una cartera d'accions
* actualizarà els valor de les carteres al preu vigent de l'acció.
* s'aprofita la sobrecàrrega de mètodes
*
*/
procedure prc_cartera_actualitzar (
	rsp OUT varchar2
	) IS

 -- declaració de variables locals del procediment
	prc_nom varchar2(25 CHAR):='PRC_CARTERA_ACTUALITZAR';
	prc_paramIN varchar(100 CHAR) := null;
	sError varchar(512);
	var_valor_accio accions.valorvigent%type;
	comptador number (4) := 0;
	capCartera EXCEPTION; -- es defineix una excepció pròpia

--cursor amb les carteres associades a client i accions actives
	cursor cursorCartera
	IS
		select ca.idcartera, ca.client, ca.accio, ca.quantitat
			from cartera ca 
				join accions ac
				  on ac.idaccio = ca.accio
				join clients cl
				  on cl.idclient = ca.client
			where ac.esactiu = 'Y' and cl.esactiu = 'Y'
			for update of valorTotal;
	
	cursor cursorAccions (pIdaccio accions.idaccio%type)
	IS
		select a.valorVigent 
		from accions a
			join cartera c 
				on c.accio = a.idaccio
			where a.esactiu = 'Y' and a.idAccio = pIdaccio;

BEGIN

-->>recull dels paràmetres d'entrada a un string
	prc_paramIN := 'null ';
--<<fi del recull dels paràmetres d'entrada

	FOR var_cartera IN cursorCartera 
	LOOP
		OPEN cursorAccions(var_cartera.accio);
		fetch cursorAccions into var_valor_accio;
		DBMS_OUTPUT.PUT_LINE('auto '||var_cartera.idcartera||', '||var_valor_accio );

		UPDATE cartera
			SET valorTotal = var_cartera.quantitat*var_valor_accio
			where CURRENT OF cursorCartera;

		close cursorAccions;
		comptador := comptador + 1;
	END LOOP;

	if comptador = 0 THEN
		raise capCartera;
	END IF;



--registra a la taula Log l'execució del procediment
	pkg_control.prc_logProcediment(prc_nom, prc_paramIN, 'FI OK');
--s'informa del paràmetre de sortida
	rsp := 'OK';

	EXCEPTION
		WHEN capCartera THEN
			if cursorCartera%isopen THEN
				close cursorCartera;
			end if;
			
			sError := pkg_control.fun_error(4)||substr(SQLERRM, 1, 500);
	--registra a la taula Log l'execució del procediment
			pkg_control.prc_logProcediment(prc_nom, prc_paramIN, sError);
	-- s'informa del paràmetre de sortida
			rsp := sError;


		WHEN NO_DATA_FOUND THEN
			sError := pkg_control.fun_error(2)||substr(SQLERRM, 1, 500);
--registra a la taula Log l'execució del procediment
			pkg_control.prc_logProcediment(prc_nom, prc_paramIN, sError);
-- s'informa del paràmetre de sortida
			rsp := sError;

		WHEN others THEN
			sError := pkg_control.fun_error(1)||substr(SQLERRM, 1, 500);
--registra a la taula Log l'execució del procediment
			pkg_control.prc_logProcediment(prc_nom, prc_paramIN, sError);
-- s'informa del paràmetre de sortida
			rsp := sError;

END prc_cartera_actualitzar;



-- fi del paquet
END pkg_cartera;
/

spool off;