/* 
* Paquet clients
* conté els mètodes: Alta, modificació, baixa lògica
*
*/

spool ./pkg_clients.log;
--capçalera paquet
create or replace package pkg_clients is

--procediment per donar d'alta un client
procedure prc_client_alta (
	pNomclient VARCHAR2, 
	pCognoms VARCHAR2, 
	pNif VARCHAR2, 
	pCorreuelectronic VARCHAR2, 
	pTelefon VARCHAR2,
	rsp OUT varchar2
	);
	
--procediment per modificar les dades principals d'un clients, donat el seu id
procedure prc_client_modificacio (
	pIdclient NUMBER,
	pNomclient VARCHAR2, 
	pCognoms VARCHAR2, 
	pNif VARCHAR2, 
	pCorreuelectronic VARCHAR2, 
	pTelefon VARCHAR2,
	rsp OUT varchar2
	);

--procediment per donar de baixa lògica un client amb id passat per paràmetre
procedure prc_client_baixa (
	pIdclient NUMBER,
	rsp OUT varchar2
	);

--procediment per donar de baixa lògica un client amb el nif passat per paràmetre
procedure prc_client_baixa (
	pNif VARCHAR2,
	rsp OUT varchar2
	);
	
end pkg_clients;
/

-- cos del paquet
CREATE OR REPLACE PACKAGE BODY pkg_clients is

/*
* definicio del procediment d'alta de clients
*
*/
PROCEDURE prc_client_alta (
	pNomclient VARCHAR2, 
	pCognoms VARCHAR2, 
	pNif VARCHAR2, 
	pCorreuelectronic VARCHAR2, 
	pTelefon VARCHAR2,
	rsp OUT varchar2
	) IS
-- declaració de variables locals del procediment
	prc_nom varchar2(25 CHAR):='PRC_CLIENT_ALTA';
	prc_paramIN varchar(100 CHAR) := null;
	sError varchar(512);
BEGIN
-->>recull dels paràmetres d'entrada a un string
	prc_paramIN := pNomclient||', '||pCognoms||', '||pNif||', '||
		pCorreuelectronic||', '||pTelefon;
--<<fi del recull dels paràmetres d'entrada


--inserció a la taula CLIENTS del nou registre amb les dades passades
	INSERT INTO clients (idclient, nomclient, cognoms, nif, correuelectronic, telefon)
		VALUES (clients_seq.nextval, UPPER(pNomclient), UPPER(pCognoms), UPPER(pNif), pCorreuelectronic, pTelefon );

--registra a la taula Log l'execució del procediment
	pkg_control.prc_logProcediment(prc_nom, prc_paramIN, 'FI OK');
--s'informa del paràmetre de sortida
	rsp := 'OK';

	EXCEPTION
		WHEN DUP_VAL_ON_INDEX THEN
			sError := pkg_control.fun_error(3)||substr(SQLERRM, 1, 500);
--registra a la taula Log l'execució del procediment
			pkg_control.prc_logProcediment(prc_nom, prc_paramIN, sError);
-- s'informa del paràmetre de sortida
		rsp := sError;

		WHEN others THEN
			sError :=  pkg_control.fun_error(1)||substr(SQLERRM, 1, 500);
--registra a la taula Log l'execució del procediment
			pkg_control.prc_logProcediment(prc_nom, prc_paramIN, sError);
-- s'informa del paràmetre de sortida
	rsp := sError;
			
		
END prc_client_alta;

/*
* procediment per modificar les dades principals d'un clients, donat el seu id
*
*/

procedure prc_client_modificacio (
	pIdclient NUMBER,
	pNomclient VARCHAR2, 
	pCognoms VARCHAR2, 
	pNif VARCHAR2, 
	pCorreuelectronic VARCHAR2, 
	pTelefon VARCHAR2,
	rsp OUT varchar2
	) is

-- declaració de variables locals del procediment
	prc_nom varchar2(25 CHAR):='PRC_CLIENT_MODIFICACIO';
	prc_paramIN varchar(100 CHAR) := null;
	sError varchar(512);
	var_client_local clients%rowtype;

BEGIN

-->>recull dels paràmetres d'entrada a un string
	prc_paramIN := pIdclient||', '||pNomclient||', '||pCognoms||', '||pNif||', '||
		pCorreuelectronic||', '||pTelefon;
--<<fi del recull dels paràmetres d'entrada
	
	if pIdclient is null THEN
		raise admin_tfc.pkg_control.insert_null;
	end if;

--es sel.lecciona el client amb el idclient passat per paràmetre
-- i es modificació del registre  de la taula clients amb les noves dades passades
  SELECT * 
	INTO var_client_local
	from clients c
	where c.idclient = pIdclient;
--si el paràmetre d'entrada no és null s'actualitza el registre, si és null es manté el valor anterior
	var_client_local.nomClient := NVL(pNomClient, var_client_local.nomClient);
	var_client_local.cognoms := NVL (pCognoms, var_client_local.cognoms);
	var_client_local.nif := NVL (pNif, var_client_local.nif);
	var_client_local.Correuelectronic := NVL (pCorreuelectronic, var_client_local.Correuelectronic);
	var_client_local.telefon := NVL (pTelefon, var_client_local.telefon);

	--DBMS_OUTPUT.PUT_LINE ('fitxer trobat '|| pIdclient||' '||pNomClient);
	
--actualització de la taula clients amb els nous valors	
	UPDATE clients 
		set row = var_client_local 
		where idclient = pIdclient;
	
	
-- visualització d resultats
/*	FOR item IN (
		SELECT * FROM clients
	)
	LOOP
		DBMS_OUTPUT.PUT_LINE 
		('nom = ' || item.nomClient || ', cognoms = ' || item.cognoms || ', nif=' ||item.nif);
	END LOOP;*/
	

--registra a la taula Log l'execució del procediment
	pkg_control.prc_logProcediment(prc_nom, prc_paramIN, 'FI OK');
--s'informa del paràmetre de sortida
	rsp := 'OK';

 	EXCEPTION
		WHEN NO_DATA_FOUND THEN
			sError := pkg_control.fun_error(2)||substr(SQLERRM, 1, 500);
--registra a la taula Log l'execució del procediment
			pkg_control.prc_logProcediment(prc_nom, prc_paramIN, sError);
-- s'informa del paràmetre de sortida
			rsp := sError;
	WHEN DUP_VAL_ON_INDEX THEN
			sError := pkg_control.fun_error(3)||substr(SQLERRM, 1, 500);
--registra a la taula Log l'execució del procediment
			pkg_control.prc_logProcediment(prc_nom, prc_paramIN, sError);
-- s'informa del paràmetre de sortida
			rsp := sError;
	WHEN others THEN
			sError := pkg_control.fun_error(1)||substr(SQLERRM, 1, 500);
--registra a la taula Log l'execució del procediment
			pkg_control.prc_logProcediment(prc_nom, prc_paramIN, sError);
-- s'informa del paràmetre de sortida
			rsp := sError;


end prc_client_modificacio;

/*
* procediment per donar de baixa lògica un client amb id passat per paràmetre
* es sobrecarregarà amb un altre paquet per nif
* 
*/
procedure prc_client_baixa (
	pIdclient NUMBER,
	rsp OUT varchar2
	) is

-- declaració de variables locals del procediment
	prc_nom varchar2(25 CHAR):='PRC_CLIENT_BAIXA';
	prc_paramIN varchar(100 CHAR) := null;
	sError varchar(512);
	comptador number(4);
BEGIN
-->>recull dels paràmetres d'entrada a un string
	prc_paramIN := pIdclient;
--<<fi del recull dels paràmetres d'entrada
	
	if pIdclient is null THEN
		raise admin_tfc.pkg_control.insert_null;
	end if;

	UPDATE ORDRES O
	SET estat ='CA'
	WHERE O.CLIENT = pIdclient;
	
	UPDATE clients c
	SET esactiu = 'N'
	WHERE c.idclient = pIdclient;
	comptador := SQL%ROWCOUNT;
	
	IF comptador >= 1 THEN
	--registra a la taula Log l'execució del procediment
		pkg_control.prc_logProcediment(prc_nom, prc_paramIN, 'FI OK');
	--s'informa del paràmetre de sortida
		rsp := 'OK';
	ELSE
		RAISE NO_DATA_FOUND;

	END IF;

	EXCEPTION
		WHEN NO_DATA_FOUND THEN
			sError :=pkg_control.fun_error(4)||substr(SQLERRM, 1, 500);
--registra a la taula Log l'execució del procediment
			pkg_control.prc_logProcediment(prc_nom, prc_paramIN, sError);
-- s'informa del paràmetre de sortida
			rsp := sError;
		WHEN others THEN
			sError := pkg_control.fun_error(1)||substr(SQLERRM, 1, 500);
--registra a la taula Log l'execució del procediment
			pkg_control.prc_logProcediment(prc_nom, prc_paramIN, sError);
-- s'informa del paràmetre de sortida
			rsp := sError;


end prc_client_baixa;


/*
*procediment per donar de baixa lògica un client amb el nif passat per paràmetre
* es sobrecarregarà amb un altre paquet per idclient
*
*/

procedure prc_client_baixa (
	pNif VARCHAR2,
	rsp OUT varchar2
	) is

-- declaració de variables locals del procediment
	prc_nom varchar2(25 CHAR):='PRC_CLIENT_BAIXA';
	prc_paramIN varchar(100 CHAR) := null;
	sError varchar(512);
	comptador number(4);
BEGIN
-->>recull dels paràmetres d'entrada a un string
	prc_paramIN := pNif;
--<<fi del recull dels paràmetres d'entrada
	
	
	if pNif is null THEN
		raise admin_tfc.pkg_control.insert_null;
	end if;

	UPDATE ORDRES O
	SET estat ='CA'
	WHERE O.CLIENT = (select idclient from clients C
			where c.nif = UPPER(pNif));

	UPDATE clients c
	SET esactiu = 'N'
	WHERE c.idclient = (select idclient from clients C
			where c.nif =  UPPER(pNif));
	comptador := SQL%ROWCOUNT;
	
	
	IF comptador >= 1 THEN
	--registra a la taula Log l'execució del procediment
		pkg_control.prc_logProcediment(prc_nom, prc_paramIN, 'FI OK');
	--s'informa del paràmetre de sortida
		rsp := 'OK';
	ELSE
		RAISE NO_DATA_FOUND;

	END IF;

	EXCEPTION
		WHEN NO_DATA_FOUND THEN
			sError := pkg_control.fun_error(4)||substr(SQLERRM, 1, 500);
--registra a la taula Log l'execució del procediment
			pkg_control.prc_logProcediment(prc_nom, prc_paramIN, sError);
-- s'informa del paràmetre de sortida
			rsp := sError;
		WHEN others THEN
			sError := pkg_control.fun_error(1)||substr(SQLERRM, 1, 500);
--registra a la taula Log l'execució del procediment
			pkg_control.prc_logProcediment(prc_nom, prc_paramIN, sError);
-- s'informa del paràmetre de sortida
			rsp := sError;


end prc_client_baixa;



-- fi del paquet
end pkg_clients;
/
spool off;

