/* 
* Paquet consultes 
* conté les consultes a la BD d'explotació
*
*/

spool ./pkg_consultes.log;
--capçalera paquet
create or replace package pkg_consultes is

-- procediment que executa la consulta que retorna les dades de clients que han realitzat operacions durant l'any
procedure prc_cns_operacionsClient (
	pAny varchar2, 
	rsp OUT varchar2
	);

-- procediment que executa la consulta que retorna les dades de accions que han participat en operacions durant l'any
procedure prc_cns_operacionsAccio (
	pAny varchar2, 
	rsp OUT varchar2
	);

-- procediment que executa la consulta que retorna les 10 accions més adquirides per sistemes automàtics
procedure prc_cns_10accionsMesAdquirides (
	rsp OUT varchar2
	);

-- procediment que executa la consulta que retorna la composicio de la artera de valors d'un client a una data concreta
procedure prc_cns_carteraVigent (
	pIdClient NUMBER,
	pData DATE,
	rsp OUT varchar2
	);

END pkg_consultes;
/

-- cos del paquet
CREATE OR REPLACE PACKAGE BODY pkg_consultes is

/**
* Definició del procedeiment que executa una consulta que retorna el llistat de tots els clients que han 
* realitzat operacions en un any concret, passat per paràmetre. Es considera operació qualsevol ordre oberta, ja sigui
* de compra o venda, en qualsevol estat.
*/
procedure prc_cns_operacionsClient (
	pAny varchar2, 
	rsp OUT varchar2
	) IS

-- declaració de variables locals del procediment
	prc_nom varchar2(40 CHAR):='PRC_CNS_OPERACIONS_ANUALS_CLIENT';
	prc_paramIN varchar(100 CHAR) := null;
	sError varchar(512);
	
	var_comptador NUMBER(4) :=0;


-- llista de tots els clients que han realitzat operacions aquell any
	CURSOR cursorlltaClients
		IS
			select distinct client
				from ordres
				where extract(year from dataexecucio) = pAny;

	reg_lltaClient cursorlltaClients%rowtype;

type reg_DadesClient IS RECORD (
	pr_idclient NUMBER(6),
	pr_nomClient varchar2 (20),
	pr_numOrCompra NUMBER (4),
	pr_numOrVenda  NUMBER (4),
	pr_numOrCompraEX  NUMBER (4),
	pr_numOrVendaEX  NUMBER (4),
	pr_valorOrCompra  NUMBER (6),
	pr_valorOrVenda  NUMBER (6),
	pr_valorMigCompra  NUMBER (6,2),
	pr_valorMigVenda  NUMBER (6,2),
	pr_numOrCompraCA  NUMBER (4),
	pr_numOrVendaCA  NUMBER (4)
);

type t_dadesClient IS TABLE OF reg_DadesClient INDEX BY BINARY_INTEGER ;

llista_clients t_dadesClient;



BEGIN


-->>recull dels paràmetres d'entrada a un string
	prc_paramIN := pAny;
--<<fi del recull dels paràmetres d'entrada

	OPEN cursorlltaClients;
	FETCH cursorlltaClients INTO reg_lltaClient;
		IF cursorlltaClients%notfound THEN
			raise NO_DATA_FOUND;
		end if;
	WHILE cursorlltaClients%FOUND LOOP
		
		var_comptador := var_comptador + 1;

	-- id del client
		llista_clients(var_comptador).pr_idclient := reg_lltaClient.client;

	--nom del client
		select nomclient
		into llista_clients(var_comptador).pr_nomClient
		from clients
		where idclient = reg_lltaClient.client;
	--	dbms_output.put_line('nom:'||llista_clients(var_comptador).pr_nomClient);
		

	-- nombre ordres de compra
		select count(*)
		INTO llista_clients(var_comptador).pr_numOrCompra
		from ordres
		where tipus = 'COMPRA' AND client = reg_lltaClient.client  AND extract(year from dataentrada) = pAny;
	--	dbms_output.put_line('num c:'||llista_clients(var_comptador).pr_numOrCompra);

	-- nombre ordres de venda
		select count(*)
		INTO llista_clients(var_comptador).pr_numOrVenda
		from ordres
		where tipus = 'VENDA' AND client = reg_lltaClient.client AND extract(year from dataentrada) = pAny;
	--	dbms_output.put_line('num v:'||llista_clients(var_comptador).pr_numOrVenda);

	-- nombre ordres de compra executades
		select count(*)
		INTO llista_clients(var_comptador).pr_numOrCompraEX
		from ordres
		where tipus = 'COMPRA' and estat ='EX' and client = reg_lltaClient.client AND extract(year from dataentrada) = pAny;
	--	dbms_output.put_line('num cex:'||llista_clients(var_comptador).pr_numOrCompraEX);

	-- nombre ordres de venda executades
		select count(*)
		INTO llista_clients(var_comptador).pr_numOrVendaEX
		from ordres
		where tipus = 'VENDA' and estat ='EX' and client = reg_lltaClient.client AND extract(year from dataentrada) = pAny;
	--	dbms_output.put_line('num vex:'||llista_clients(var_comptador).pr_numOrVendaEX);

	-- nombre ordres de compra cancelades
		select count(*)
		INTO llista_clients(var_comptador).pr_numOrCompraCA
		from ordres
		where tipus = 'COMPRA' and estat ='CA' and client = reg_lltaClient.client AND extract(year from dataentrada) = pAny;
	--	dbms_output.put_line('num cca:'||llista_clients(var_comptador).pr_numOrCompraEX);

	-- nombre ordres de venda cancelades
		select count(*)
		INTO llista_clients(var_comptador).pr_numOrVendaCA
		from ordres
		where tipus = 'VENDA' and estat ='CA' and client = reg_lltaClient.client AND extract(year from dataentrada) = pAny;
--		dbms_output.put_line('num vca:'||llista_clients(var_comptador).pr_numOrVendaEX);

	-- sumatori en € de les compres realitzades al llarg de l'any
		select sum(total)
		INTO llista_clients(var_comptador).pr_valorOrCompra
		from ordres 
		where client = reg_lltaClient.client and estat='EX' and tipus ='COMPRA' AND extract(year from dataentrada) = pAny;
--		dbms_output.put_line('sum c:'||llista_clients(var_comptador).pr_valorOrCompra);


	-- sumatori en € de les vendes realitzades al llarg de l'any
		select sum(total)
		INTO llista_clients(var_comptador).pr_valorOrVenda
		from ordres 
		where client = reg_lltaClient.client and estat='EX' and tipus ='VENDA' AND extract(year from dataentrada) = pAny;
--		dbms_output.put_line('sum v:'||llista_clients(var_comptador).pr_valorOrVenda);

	-- calcul del promig de les operacions de compra
		IF (llista_clients(var_comptador).pr_numOrCompraEX != 0 ) then
			llista_clients(var_comptador).pr_valorMigCompra := llista_clients(var_comptador).pr_valorOrCompra / llista_clients(var_comptador).pr_numOrCompraEX;
		ELSE
			llista_clients(var_comptador).pr_valorMigCompra := 0;
		END IF;
	-- calcul del promig de les operacions de venda
		IF (llista_clients(var_comptador).pr_numOrVendaEX != 0 ) then
			llista_clients(var_comptador).pr_valorMigVenda := llista_clients(var_comptador).pr_valorOrVenda / llista_clients(var_comptador).pr_numOrVendaEX;
		ELSE
			llista_clients(var_comptador).pr_numOrVendaEX := 0;
		END IF;

		FETCH cursorlltaClients INTO reg_lltaClient;
	END LOOP;
	CLOSE cursorlltaClients;

--	 dbms_output.put_line('num : '||llista_clients.count );
	
/*	FOR nom In cursorlltaClients
	LOOP
		dbms_output.put_line('nom: '||nom.client);
	END LOOP;*/

	
		
	

	FOR v_conta IN 1 ..llista_clients.count 
	LOOP
      dbms_output.put_line(
    	'idClient: ' || llista_clients(v_conta).pr_idclient 
		||', NomClient: '|| llista_clients(v_conta).pr_nomClient
		||', NumOrCompra: '|| llista_clients(v_conta).pr_numOrCompra 
		||', NumOrVenda: '|| llista_clients(v_conta).pr_numOrVenda 
		||', numOrCompraEX: '||llista_clients(v_conta).pr_numOrCompraEX
		||', numOrVendaEX: '||llista_clients(v_conta).pr_numOrVendaEX
		||', valorOrCompra: '||llista_clients(v_conta).pr_valorOrCompra
		||', valorOrVenda: '||llista_clients(v_conta).pr_valorOrVenda
		||', valorMigCompra: '||llista_clients(v_conta).pr_valorMigCompra
		||', valorMigVenda: '||llista_clients(v_conta).pr_valorMigVenda
		||', numOrCompraCA: '||llista_clients(v_conta).pr_numOrCompraCA
		||', numOrVendaCA: '||llista_clients(v_conta).pr_numOrVendaCA
	);
    
   	END LOOP;


   	--registra a la taula Log l'execució del procediment
	pkg_control.prc_logProcediment(prc_nom, prc_paramIN, 'FI OK');
--s'informa del paràmetre de sortida
	rsp := 'OK';

	EXCEPTION
		WHEN NO_DATA_FOUND THEN
			IF cursorlltaClients%ISOPEN THEN
				CLOSE cursorlltaClients;
 	      	END IF;
			sError := pkg_control.fun_error(2)||substr(SQLERRM, 1, 500);
--registra a la taula Log l'execució del procediment
			pkg_control.prc_logProcediment(prc_nom, prc_paramIN, sError);
-- s'informa del paràmetre de sortida
			rsp := sError;

		WHEN others THEN
			sError :=  pkg_control.fun_error(1)||substr(SQLERRM, 1, 500);
--registra a la taula Log l'execució del procediment
			pkg_control.prc_logProcediment(prc_nom, prc_paramIN, sError);
-- s'informa del paràmetre de sortida
			rsp := sError;

	

END prc_cns_operacionsClient;


/**
* Definició del procediment que executa una consulta que retorna el llistat de totes les accions sobre les que 
* s'ha executat alguna operació, ja sigui de compra o venda. Han d'estar associades a ordres executades.
*/
procedure prc_cns_operacionsAccio (
	pAny varchar2, 
	rsp OUT varchar2
	) IS

-- declaració de variables locals del procediment
	prc_nom varchar2(40 CHAR):='PRC_CNS_OPERACIONS_ANUALS_ACCIO';
	prc_paramIN varchar(100 CHAR) := null;
	sError varchar(512);
	
	var_comptador NUMBER(4) :=0;
	var_valor1 NUMBER (6);
	var_valor2 NUMBER(6);
	
-- llista de totes les accions que participen en ordres executades durant aquell any
	CURSOR cursorlltaAccions
		IS
			select distinct accio
				from ordres
				where estat = 'EX' and extract(year from dataexecucio) = pAny
				order by accio;

	reg_lltaAccio cursorlltaAccions%ROWTYPE;

	type reg_DadesAccio IS RECORD (
	pr_idAccio NUMBER(6),
	pr_nomAccio varchar2 (50),
	pr_numOrCompraEX  NUMBER (4),
	pr_numOrVendaEX  NUMBER (4),
	pr_valorOrCompra  NUMBER (6),
	pr_valorOrVenda  NUMBER (6),
	pr_valorMigCompra  NUMBER (6,2),
	pr_valorMigVenda  NUMBER (6,2),
	pr_revaloritzacio  NUMBER (5,2)
);

type t_dadesAccio IS TABLE OF reg_DadesAccio INDEX BY BINARY_INTEGER ;

llista_accions t_dadesAccio;



BEGIN


-->>recull dels paràmetres d'entrada a un string
	prc_paramIN := pAny;
--<<fi del recull dels paràmetres d'entrada

	OPEN cursorlltaAccions;
	FETCH cursorlltaAccions INTO reg_lltaAccio;
		IF cursorlltaAccions%notfound THEN
			raise NO_DATA_FOUND;
		end if;
	WHILE cursorlltaAccions%FOUND LOOP
		
		var_comptador := var_comptador + 1;

-- id de la accio
		llista_accions(var_comptador).pr_idAccio := reg_lltaAccio.accio;

--nom de la accio
		select nomaccio
		into llista_accions(var_comptador).pr_nomAccio
		from accions
		where idAccio = reg_lltaAccio.accio;
--		dbms_output.put_line('nom:'||llista_accions(var_comptador).pr_nomAccio);

-- nombre d'operacions de compra executades d'aquella acció
		select count(*)
		INTO llista_accions(var_comptador).pr_numOrCompraEX
		from ordres
		where tipus = 'COMPRA' and estat ='EX' and accio = reg_lltaAccio.accio AND extract(year from dataexecucio) = pAny;
--		dbms_output.put_line('num cex:'||llista_accions(var_comptador).pr_numOrCompraEX);

-- nombre d'operacions de venda executades d'aquella acció
		select count(*)
		INTO llista_accions(var_comptador).pr_numOrVendaEX
		from ordres
		where tipus = 'VENDA' and estat ='EX' and accio = reg_lltaAccio.accio AND extract(year from dataexecucio) = pAny;
--		dbms_output.put_line('num vex:'||llista_accions(var_comptador).pr_numOrVendaEX);
-- suma en € de les ordres de compra executades d'aquella acció
		select sum(total)
		INTO llista_accions(var_comptador).pr_valorOrCompra
		from ordres 
		where accio = reg_lltaAccio.accio and estat='EX' and tipus ='COMPRA' AND extract(year from dataexecucio) = pAny;
--		dbms_output.put_line('sum c:'||llista_accions(var_comptador).pr_valorOrCompra);

-- suma en € de les ordres de venda executades d'aquella acció
		select sum(total)
		INTO llista_accions(var_comptador).pr_valorOrVenda
		from ordres 
		where accio = reg_lltaAccio.accio and estat='EX' and tipus ='VENDA' AND extract(year from dataexecucio) = pAny;
--		dbms_output.put_line('sum v:'||llista_accions(var_comptador).pr_valorOrVenda);
-- valor mitjà de les ordres de compra executades
		IF (llista_accions(var_comptador).pr_numOrCompraEX != 0 ) then
			llista_accions(var_comptador).pr_valorMigCompra := 
			llista_accions(var_comptador).pr_valorOrCompra / llista_accions(var_comptador).pr_numOrCompraEX;
		ELSE
			llista_accions(var_comptador).pr_valorMigCompra := 0;
		END IF;
--		dbms_output.put_line('mitja c:'||llista_accions(var_comptador).pr_valorMigCompra);
-- valor mitja de les ordres de venda executades
		IF (llista_accions(var_comptador).pr_numOrVendaEX != 0 ) then
			llista_accions(var_comptador).pr_valorMigVenda := llista_accions(var_comptador).pr_valorOrVenda / llista_accions(var_comptador).pr_numOrVendaEX;
		ELSE
			llista_accions(var_comptador).pr_valorMigVenda := 0;
		END IF;
--		dbms_output.put_line('mitja v:'||llista_accions(var_comptador).pr_valorMigVenda);
-- revalorització en % des del 1 de gener del any indicat.
-- a la pràctica s'agafen per comparar el 1er valor entrat de l'any i el últim valor entrat de l'any de la consulta
	
		
		BEGIN
			select valorNou
				into var_valor1
				from historicaccions
				where accio = reg_lltaAccio.accio 
					and datainici = (select min(datainici) from historicaccions 
										where extract(year from datainici) = pAny) 
					and rownum <=1
				order by datainici
			;
		EXCEPTION
			WHEN NO_DATA_FOUND THEN
				var_valor1 := 0;
		END;


		BEGIN
			select valorNou
				into var_valor2
				from historicaccions
				where accio = reg_lltaAccio.accio 
					and  datainici =  (select max(datainici) from historicaccions 
									where extract(year from datainici) = pAny)
					and rownum <=1
			;
		
		EXCEPTION
			WHEN NO_DATA_FOUND THEN
				var_valor2 := 0;
		END;


		IF (var_valor1 != 0) THEN
			 llista_accions(var_comptador).pr_revaloritzacio := (var_valor2 - var_valor1) / var_valor1 *100;
		ELSE 
			llista_accions(var_comptador).pr_revaloritzacio := 0;
		END IF;

--		dbms_output.put_line('valor 1: '||var_valor1||' valor 2: '||var_valor2||' reval : '||llista_accions(var_comptador).pr_revaloritzacio );

	FETCH cursorlltaAccions INTO reg_lltaAccio;
	END LOOP;
	CLOSE cursorlltaAccions;


	FOR v_conta IN 1 ..llista_accions.count 
		LOOP
	      dbms_output.put_line(
	    	'idAccio: ' || llista_accions(v_conta).pr_idAccio 
			||', NomAccio: '|| llista_accions(v_conta).pr_nomAccio
			||', numOrCompraEX: '||llista_accions(v_conta).pr_numOrCompraEX
			||', numOrVendaEX: '||llista_accions(v_conta).pr_numOrVendaEX
			||', valorOrCompra: '||llista_accions(v_conta).pr_valorOrCompra
			||', valorOrVenda: '||llista_accions(v_conta).pr_valorOrVenda
			||', valorMigCompra: '||llista_accions(v_conta).pr_valorMigCompra
			||', valorMigVenda: '||llista_accions(v_conta).pr_valorMigVenda
			||', revaloritzacio: '||llista_accions(v_conta).pr_revaloritzacio
			);
	    
	   	END LOOP;

--registra a la taula Log l'execució del procediment
	pkg_control.prc_logProcediment(prc_nom, prc_paramIN, 'FI OK');
--s'informa del paràmetre de sortida
	rsp := 'OK';

	EXCEPTION
		WHEN NO_DATA_FOUND THEN
			IF cursorlltaAccions%ISOPEN THEN
				CLOSE cursorlltaAccions;
 	      	END IF;
			sError := pkg_control.fun_error(2)||substr(SQLERRM, 1, 500);
--registra a la taula Log l'execució del procediment
			pkg_control.prc_logProcediment(prc_nom, prc_paramIN, sError);
-- s'informa del paràmetre de sortida
			rsp := sError;

		WHEN others THEN
			sError :=  pkg_control.fun_error(1)||substr(SQLERRM, 1, 500);
--registra a la taula Log l'execució del procediment
			pkg_control.prc_logProcediment(prc_nom, prc_paramIN, sError);
-- s'informa del paràmetre de sortida
			rsp := sError;
	

END prc_cns_operacionsAccio;



/**
* Definició del procediment que executa una consulta que retorna el llistat de les 10 accions més adquirides
* per sistemes automàtics, ordenades de forma decreixent pel seu volum
*/
procedure prc_cns_10accionsMesAdquirides (
	rsp OUT varchar2
	) IS

-- declaració de variables locals del procediment
	prc_nom varchar2(40 CHAR):='PRC_CNS_10ACC_MES_ADQUIRIDES';
	prc_paramIN varchar(100 CHAR) := null;
	sError varchar(512);

	type reg_10AccMesAdqurides IS RECORD (
		pr_idAccio NUMBER(6),
		pr_nomAccio varchar2 (50 CHAR),
		pr_volum  NUMBER (6)
	);

	type t_10AccMes IS TABLE OF reg_10AccMesAdqurides; 

	llista_10AccMes t_10AccMes;

	CURSOR cursorLlista10mes
	IS 
		select *
		from (select o1.accio, ac.nomaccio, sum(quantitat) as volum
		      from ordres o1
		      JOIN
          		ACCIONS ac
          		ON o1.accio = ac.idaccio
		      where o1.origen = 'AUT' and o1.estat = 'EX'
		      group by o1.accio, ac.nomaccio
		      order by volum desc)
		where rownum <=10;


BEGIN

-->>recull dels paràmetres d'entrada a un string
	prc_paramIN := NULL;
--<<fi del recull dels paràmetres d'entrada

-- lista de les 10 accions més adquirides pels sistemes automàtics

	OPEN cursorLlista10mes;
	FETCH cursorLlista10mes BULK COLLECT INTO llista_10AccMes limit 10;
		if llista_10AccMes.count = 0 THEN
			RAISE NO_DATA_FOUND;
		end if;
		
	CLOSE cursorLlista10mes;

	FOR v_conta in 1 ..llista_10AccMes.count 
	LOOP
		dbms_output.put_line(
	    	'idAccio: ' || llista_10AccMes(v_conta).pr_idAccio
	    	 ||', nom: '|| llista_10AccMes(v_conta).pr_nomAccio
			||', volum: '|| llista_10AccMes(v_conta).pr_volum
		);
	END LOOP;

--registra a la taula Log l'execució del procediment
	pkg_control.prc_logProcediment(prc_nom, prc_paramIN, 'FI OK');
--s'informa del paràmetre de sortida
	rsp := 'OK';

	EXCEPTION
		WHEN NO_DATA_FOUND THEN
			IF cursorLlista10mes%ISOPEN THEN
				CLOSE cursorLlista10mes;
 	      	END IF;
			sError := pkg_control.fun_error(2)||substr(SQLERRM, 1, 500);
--registra a la taula Log l'execució del procediment
			pkg_control.prc_logProcediment(prc_nom, prc_paramIN, sError);
-- s'informa del paràmetre de sortida
			rsp := sError;

		WHEN others THEN
			sError :=  pkg_control.fun_error(1)||substr(SQLERRM, 1, 500);
--registra a la taula Log l'execució del procediment
			pkg_control.prc_logProcediment(prc_nom, prc_paramIN, sError);
-- s'informa del paràmetre de sortida
			rsp := sError;

	

END prc_cns_10accionsMesAdquirides;

/**
* Definició del procediment que executa una consulta que retorna el llistat amb la cartera d'accions d'un client
* vigent en la data especificada
*/
procedure prc_cns_carteraVigent (
	pIdClient NUMBER,
	pData DATE,
	rsp OUT varchar2
	) IS

-- declaració de variables locals del procediment
	prc_nom varchar2(40 CHAR):='PRC_CNS_CARTERA_VIGENT';
	prc_paramIN varchar(100 CHAR) ;
	sError varchar(512);
	sumaTotal NUMBER(6):= 0;
	dataIncorrecte EXCEPTION;

-- llista de les accions lligades al client ( cartera) abans de la data indicada
	CURSOR cursorAccions (cpIdClient NUMBER, cpData DATE)
	IS
		select distinct ca.accio 
  			from historicCartera hc
  			join cartera ca on hc.cartera = ca.idcartera
 			where ca.client = cpIdClient and hc.datainici <= cpData;

	reg_lltaAccio cursorAccions%rowtype;

-- llista del històric de la cartera d'accions lligada a client en una data concreta
	CURSOR cursorHistoricCartera (cpIdClient NUMBER, cpIdAccio NUMBER, cpData DATE)
	IS
		select cons1.*
		from (select hc1.quantitat, hc1.valortotal, hc1.datainici from historicCartera hc1
                            join cartera ca1 on hc1.cartera = ca1.idcartera
                            where ca1.client = cpIdClient and ca1.accio = cpIdAccio ) cons1
                            
		join (select max(datainici) as maxdata  
          		from (select datainici from historicCartera hc2
                            join cartera ca2 on hc2.cartera = ca2.idcartera
                            where ca2.client = cpIdClient and ca2.accio = cpIdAccio and hc2.datainici <= cpData) cons3) cons2
		on cons1.datainici = cons2.maxdata;


	type reg_carteraVigent IS RECORD (
		pr_idAccio NUMBER(6),
		pr_nomAccio varchar2 (50 CHAR),
		pr_quantitat NUMBER(6), 
		pr_valortotal NUMBER(6), 
		pr_dataVigencia DATE
	);

	type t_carteraVigent IS TABLE OF reg_carteraVigent INDEX BY BINARY_INTEGER; 
	llista_carteraVigent t_carteraVigent;

	var_comptador NUMBER(4):=0;

	var_data date;

	BEGIN

-->>recull dels paràmetres d'entrada a un string
	prc_paramIN := pIdClient||', '||pData;
--<<fi del recull dels paràmetres d'entrada
	
-- recorregut per totes les accions en cartera del client
	OPEN cursorAccions (pIdClient, pData);
	FETCH cursorAccions INTO reg_lltaAccio;
		IF cursorAccions%NOTFOUND THEN
			RAISE NO_DATA_FOUND;
		END IF;
	WHILE cursorAccions%FOUND LOOP

		var_comptador := var_comptador + 1;
	
-- id de l'accio		
		llista_carteraVigent(var_comptador).pr_idAccio := reg_lltaAccio.accio;
-- nom de l'accio
		select nomAccio 
			INTO llista_carteraVigent(var_comptador).pr_nomAccio
			from accions
			where idaccio = reg_lltaAccio.accio;
-- quantitat i valor total de l'accio
		OPEN cursorHistoricCartera(pIdClient, reg_lltaAccio.accio, pData);
		FETCH cursorHistoricCartera INTO llista_carteraVigent(var_comptador).pr_quantitat, llista_carteraVigent(var_comptador).pr_valortotal,
											 llista_carteraVigent(var_comptador).pr_dataVigencia;
		CLOSE cursorHistoricCartera;

-- acumulem el valortotal de cada accio
	sumaTotal := sumaTotal + llista_carteraVigent(var_comptador).pr_valortotal;

	FETCH cursorAccions INTO reg_lltaAccio;
	END LOOP;
	CLOSE cursorAccions;

	dbms_output.put_line('idAccio'||chr(9)||'nomaccio'||chr(9)||'quantitat'||chr(9)||'valortotal'||chr(9)||'data');
	FOR v_conta in 1 ..llista_carteraVigent.count 
	LOOP
		dbms_output.put_line(llista_carteraVigent(v_conta).pr_idAccio||chr(9)||
              llista_carteraVigent(v_conta).pr_nomAccio||chr(9)||
              llista_carteraVigent(v_conta).pr_quantitat||chr(9)||
              llista_carteraVigent(v_conta).pr_valortotal||chr(9)||
              llista_carteraVigent(v_conta).pr_dataVigencia
		);
	END LOOP;
	dbms_output.put_line('Suma Total'||chr(9)||sumaTotal);

--registra a la taula Log l'execució del procediment
	pkg_control.prc_logProcediment(prc_nom, prc_paramIN, 'FI OK');
--s'informa del paràmetre de sortida
	rsp := 'OK';

	EXCEPTION
		WHEN dataIncorrecte THEN
			sError :=  pkg_control.fun_error(8)||substr(SQLERRM, 1, 500);
--registra a la taula Log l'execució del procediment
			pkg_control.prc_logProcediment(prc_nom, prc_paramIN, sError);
-- s'informa del paràmetre de sortida
			rsp := sError;

		WHEN NO_DATA_FOUND THEN
			IF cursorAccions%ISOPEN THEN
				CLOSE cursorAccions;
 	      	END IF;
			sError := pkg_control.fun_error(2)||substr(SQLERRM, 1, 500);
--registra a la taula Log l'execució del procediment
			pkg_control.prc_logProcediment(prc_nom, prc_paramIN, sError);
-- s'informa del paràmetre de sortida
			rsp := sError;

		WHEN others THEN
			sError :=  pkg_control.fun_error(1)||substr(SQLERRM, 1, 500);
--registra a la taula Log l'execució del procediment
			pkg_control.prc_logProcediment(prc_nom, prc_paramIN, sError);
-- s'informa del paràmetre de sortida
			rsp := sError;

END prc_cns_carteraVigent;


END pkg_consultes;
/
spool off;