/* 
* Paquet consultes 
* conté les consultes a la BD estadística (DWH)
*
*/

spool ./pkg_consultes_DWH.log;
--capçalera paquet
create or replace package pkg_consultes_DWH is

-- Procediment de consulta, que donat un identificador de client i un mes concret ( anterior a l'actual), 
-- retorni el benefici net, en euros, i el % de la cartera d'accions durant aquell mes.
procedure prc_cns_beneficiMesClient (
	pIdclient NUMBER, 
	pMesAny DATE,
	rsp OUT varchar2
);

-- Procediment de consulta, que donat un identificador d'una acció, retorni el nombre d'accions disponibles 
-- dins el broker en aquell moment i el nombre de clients que la tenen en la cartera d'accions de forma activa.
procedure prc_cns_accDisponibles (
	pIdAccio NUMBER,
	rsp OUT varchar2
);

-- Procediment de consulta que retorni el client amb la cartera d'accions mes gran, i el valor en euros de la cartera.
procedure prc_cns_clientCarteraMax (
	rsp OUT varchar2
);

-- Procediment de consulta que retorni l'acció més negociada ( comprada i venuda) en el broker de forma històrica, 
-- i el seu volum
procedure prc_cns_accioMesNegociada (
	rsp OUT varchar2
);

END pkg_consultes_DWH;
/

-- cos del paquet
CREATE OR REPLACE PACKAGE BODY pkg_consultes_DWH is

/**
* Definició del procediment de consulta, que donat un identificador de client i un mes concret ( anterior a l'actual), 
* retorni el benefici net, en euros, i el % de la cartera d'accions durant aquell mes.
*/
procedure prc_cns_beneficiMesClient (
	pIdclient NUMBER, 
	pMesAny DATE,
	rsp OUT varchar2
	) IS 
	
-- declaració de variables locals del procediment
	prc_nom varchar2(40 CHAR):='PRC_CNS_BENEFICI_MES_CLIENT';
	prc_paramIN varchar(100 CHAR) := null;
	sError varchar(512);
	var_Data DATE;
	var_idData NUMBER(8);
	reg_tfBeneficis tf_beneficis%rowtype;
	
BEGIN

-->>recull dels paràmetres d'entrada a un string
	prc_paramIN := pIdclient||', '||pMesAny;
--<<fi del recull dels paràmetres d'entrada
	
	-- obtenim la data d'inici de mes
	-- var_Data := to_date(pMes, 'mm');
	var_Data := pMesAny;
	

	var_idData := EXTRACT(YEAR FROM var_Data)*10000 
			+ EXTRACT(MONTH FROM var_Data)*100 +  EXTRACT(DAY FROM var_Data);
	
	select *
	into reg_tfBeneficis
    from tf_beneficis
    where idclient = pIdclient and idtemps < var_idData and rownum <=1
    order by idtemps desc;



	dbms_output.put_line('ID Client: '||reg_tfBeneficis.idclient||' Benefici net: '||reg_tfBeneficis.beneficinet
						||' % benefici: '||reg_tfBeneficis.x100benefici);


--registra a la taula Log l'execució del procediment
	pkg_control.prc_logProcediment(prc_nom, prc_paramIN, 'FI OK');
--s'informa del paràmetre de sortida
	rsp := 'OK';

	EXCEPTION
		WHEN NO_DATA_FOUND THEN
			sError := pkg_control.fun_error(2)||substr(SQLERRM, 1, 500);
--registra a la taula Log l'execució del procediment
			pkg_control.prc_logProcediment(prc_nom, prc_paramIN, sError);
-- s'informa del paràmetre de sortida
			rsp := sError;

		WHEN others THEN
			sError :=  pkg_control.fun_error(1)||substr(SQLERRM, 1, 500);
--registra a la taula Log l'execució del procediment
			pkg_control.prc_logProcediment(prc_nom, prc_paramIN, sError);
-- s'informa del paràmetre de sortida
			rsp := sError;
			
end prc_cns_beneficiMesClient;

/* Procediment de consulta, que donat un identificador d'una acció, retorni el nombre d'accions disponibles 
* dins el broker en aquell moment i el nombre de clients que la tenen en la cartera d'accions de forma activa.
*/

procedure prc_cns_accDisponibles (
	pIdAccio NUMBER,
	rsp OUT varchar2
	) IS
-- declaració de variables locals del procediment
	prc_nom varchar2(40 CHAR):='PRC_CNS_ACCIONS_DISPONIBLES';
	prc_paramIN varchar(100 CHAR) := null;
	sError varchar(512);
	reg_tf_accDisp tf_accionsDisponibles%rowtype;
	
BEGIN

-->>recull dels paràmetres d'entrada a un string
	prc_paramIN := pIdAccio;
--<<fi del recull dels paràmetres d'entrada


	select * 
	into reg_tf_accDisp
	from tf_accionsDisponibles
	where idaccio = pIdAccio and rownum = 1
	order by idtemps desc;



	dbms_output.put_line('ID Accio: '||reg_tf_accDisp.idaccio||' quantitat diponible: '||reg_tf_accDisp.numaccionsenvenda
						||' Num Clients: '||reg_tf_accDisp.numclients);


--registra a la taula Log l'execució del procediment
	pkg_control.prc_logProcediment(prc_nom, prc_paramIN, 'FI OK');
--s'informa del paràmetre de sortida
	rsp := 'OK';

	EXCEPTION
		WHEN NO_DATA_FOUND THEN
			sError := pkg_control.fun_error(2)||substr(SQLERRM, 1, 500);
--registra a la taula Log l'execució del procediment
			pkg_control.prc_logProcediment(prc_nom, prc_paramIN, sError);
-- s'informa del paràmetre de sortida
			rsp := sError;

		WHEN others THEN
			sError :=  pkg_control.fun_error(1)||substr(SQLERRM, 1, 500);
--registra a la taula Log l'execució del procediment
			pkg_control.prc_logProcediment(prc_nom, prc_paramIN, sError);
-- s'informa del paràmetre de sortida
			rsp := sError;


END prc_cns_accDisponibles;

/* Procediment de consulta que retorni el client amb la cartera d'accions mes gran, i el valor en euros de la cartera.
*
*/

procedure prc_cns_clientCarteraMax (
	rsp OUT varchar2
	) IS

-- declaració de variables locals del procediment
	prc_nom varchar2(40 CHAR):='PRC_CNS_CLIENT_AMB_MAX_CARTERA';
	prc_paramIN varchar(100 CHAR) := null;
	sError varchar(512);
	reg_tfCartera tf_carteraValors%rowtype;

BEGIN

-->>recull dels paràmetres d'entrada a un string
	prc_paramIN := null;
--<<fi del recull dels paràmetres d'entrada
	select * 
	into reg_tfCartera
	from tf_carteraValors
	where rownum = 1
	order by idtemps desc;


	dbms_output.put_line('ID Client: '||reg_tfCartera.idclient||' Num accions en cartera: '||reg_tfCartera.numaccions
						||' Valor cartera: '||reg_tfCartera.valorcartera);


--registra a la taula Log l'execució del procediment
	pkg_control.prc_logProcediment(prc_nom, prc_paramIN, 'FI OK');
--s'informa del paràmetre de sortida
	rsp := 'OK';

	EXCEPTION
		WHEN NO_DATA_FOUND THEN
			sError := pkg_control.fun_error(2)||substr(SQLERRM, 1, 500);
--registra a la taula Log l'execució del procediment
			pkg_control.prc_logProcediment(prc_nom, prc_paramIN, sError);
-- s'informa del paràmetre de sortida
			rsp := sError;

		WHEN others THEN
			sError :=  pkg_control.fun_error(1)||substr(SQLERRM, 1, 500);
--registra a la taula Log l'execució del procediment
			pkg_control.prc_logProcediment(prc_nom, prc_paramIN, sError);
-- s'informa del paràmetre de sortida
			rsp := sError;


END prc_cns_clientCarteraMax;


/* Procediment de consulta que retorni l'acció més negociada ( comprada i venuda) en el broker de forma històrica, 
* i el seu volum
*/ 

procedure prc_cns_accioMesNegociada (
	rsp OUT varchar2
	) IS

-- declaració de variables locals del procediment
	prc_nom varchar2(40 CHAR):='PRC_CNS_ACCIO_MES_NEGOCIADA';
	prc_paramIN varchar(100 CHAR) := null;
	sError varchar(512);
	reg_tfaccioMesNeg tf_acciomesnegociada%rowtype;

BEGIN

-->>recull dels paràmetres d'entrada a un string
	prc_paramIN := null;
--<<fi del recull dels paràmetres d'entrada

	select * 
	into reg_tfaccioMesNeg
	from tf_acciomesnegociada
	where rownum = 1
	order by idtemps desc;


	dbms_output.put_line('ID Accio: '||reg_tfaccioMesNeg.idAccio||' Num operacions: '||reg_tfaccioMesNeg.numOperacions);


--registra a la taula Log l'execució del procediment
	pkg_control.prc_logProcediment(prc_nom, prc_paramIN, 'FI OK');
--s'informa del paràmetre de sortida
	rsp := 'OK';

	EXCEPTION
		WHEN NO_DATA_FOUND THEN
			sError := pkg_control.fun_error(2)||substr(SQLERRM, 1, 500);
--registra a la taula Log l'execució del procediment
			pkg_control.prc_logProcediment(prc_nom, prc_paramIN, sError);
-- s'informa del paràmetre de sortida
			rsp := sError;

		WHEN others THEN
			sError :=  pkg_control.fun_error(1)||substr(SQLERRM, 1, 500);
--registra a la taula Log l'execució del procediment
			pkg_control.prc_logProcediment(prc_nom, prc_paramIN, sError);
-- s'informa del paràmetre de sortida
			rsp := sError;


END prc_cns_accioMesNegociada;

END pkg_consultes_DWH;
/

spool off;