/*
* Paquet de control
* 
*/

spool ./pkg_control.log;

-- caçalera del paquet
create or replace package pkg_control is

-- funció que inserta la informació del procediment executat a la taula Log
PROCEDURE prc_logProcediment (
	pNomprocediment VARCHAR2, 
	pParametresin VARCHAR2, 
	pFinalitzacioTxt VARCHAR2
	);

-- funció que retorna el missatge d'error personalitzat associat a un identificador
FUNCTION fun_error (
	pIdError NUMBER
	)
	RETURN VARCHAR2;

PROCEDURE prc_mercat_alta (
	pNomMercat VARCHAR2,
	rsp OUT VARCHAR2
);

insert_null EXCEPTION;
PRAGMA EXCEPTION_INIT(insert_null, -1400);
parent_key_nf EXCEPTION;
PRAGMA EXCEPTION_INIT(parent_key_nf, -2291);

end pkg_control;
/ 
-- es compila primer la capçalera i després el cos

-- cos del paquet
create or replace package body pkg_control is


-- definició del procediment d'inserció en la taula Log
PROCEDURE prc_logProcediment (
	pNomprocediment VARCHAR2, 
	pParametresin VARCHAR2, 
	pFinalitzacioTxt VARCHAR2
	) is
var_final VARCHAR2( 5 CHAR);

BEGIN
	
	var_final := substr(pFinalitzacioTxt,1, 5);

	-- inserció a la taula Log
	insert into log_procediments (idLog, nomprocediment, tempsExecucio, parametresin, finalitzacio, finalitzacioTxt)
		values (LOG_PROCEDIMENTS_SEQ.NEXTVAL, pNomprocediment, CURRENT_TIMESTAMP, pParametresin, 
				var_final, pFinalitzacioTxt);


EXCEPTION
	WHEN OTHERS THEN
		DBMS_OUTPUT.PUT_LINE('Error: no sha registrat correctament lerror');

END prc_logProcediment;



-- funció que retorna el missatge d'error personalitzat associat a un identificador
FUNCTION fun_error (
	pIdError NUMBER
	)
	RETURN VARCHAR2
	IS
-- declaració de variables locals del procediment
	var_error VARCHAR2 (100 char);

BEGIN

SELECT descripcio
	INTO var_error
	FROM ERRORS_RECONEGUTS
	WHERE idError = pIdError;

	RETURN var_error;

EXCEPTION
	WHEN NO_DATA_FOUND THEN
		DBMS_OUTPUT.PUT_LINE('Error: No sha trobat el registre');
		RETURN NULL;
	WHEN OTHERS THEN
		DBMS_OUTPUT.PUT_LINE('Error: parametre incorrecte');
END fun_error;


-- procediment per donar d'alta nous mercats
PROCEDURE prc_mercat_alta (
	pNomMercat VARCHAR2,
	rsp OUT VARCHAR2
	) IS
-- declaració de variables locals del procediment
	prc_nom varchar2(25 CHAR):='PRC_MERCAT_ALTA';
	prc_paramIN varchar(100 CHAR) := null;
	sError varchar(512);
BEGIN

-->>recull dels paràmetres d'entrada a un string
	prc_paramIN := pNomMercat;
--<<fi del recull dels paràmetres d'entrada


insert into ADMIN_TFC.MERCATS (IDMERCAT, NOMMERCAT) 
	values (admin_tfc.mercats_seq.nextval, pNomMercat);

--registra a la taula Log l'execució del procediment
	pkg_control.prc_logProcediment(prc_nom, prc_paramIN, 'FI OK');
--s'informa del paràmetre de sortida
	rsp := 'OK';	

EXCEPTION
	WHEN DUP_VAL_ON_INDEX THEN
		sError := pkg_control.fun_error(3)||substr(SQLERRM, 1, 500);
--registra a la taula Log l'execució del procediment
		pkg_control.prc_logProcediment(prc_nom, prc_paramIN, sError);
-- s'informa del paràmetre de sortida
		rsp := sError;
	WHEN insert_null THEN
		sError := pkg_control.fun_error(8)||substr(SQLERRM, 1, 500);
--registra a la taula Log l'execució del procediment
		pkg_control.prc_logProcediment(prc_nom, prc_paramIN, sError);
-- s'informa del paràmetre de sortida
		rsp := sError;
	WHEN OTHERS THEN
		sError := pkg_control.fun_error(1)||substr(SQLERRM, 1, 500);
--registra a la taula Log l'execució del procediment
		pkg_control.prc_logProcediment(prc_nom, prc_paramIN, sError);
-- s'informa del paràmetre de sortida
		rsp := sError;

end prc_mercat_alta;

end pkg_control;
/

spool off;