/* 
* Paquet dataware house, Mòdul estadístic.
* conté els mètodes: de càrrega de dimensions i de fets, i de les consultes estadístiques a les dades del DWH
*
*/

spool ./pkg_dwh.log;
--capçalera paquet
create or replace package pkg_dwh is

-- procediment que inserta un registre en la dimensió TD_Temps, d'acord a la data que se li passa per paràmetre
FUNCTION fun_insDim_tdTemps (
	pData DATE 
	)
	RETURN NUMBER;

-- procediment que inserta un registre en la dimensió TD_Accio
PROCEDURE prc_insDim_tdAccio (
	pIdAccio NUMBER
	);

-- procediment que inserta un registre en la dimensió TD_Client
PROCEDURE prc_insDim_tdClient (
	pIdClient NUMBER
	);

--procediment que inserta un registre en la taula de fets TF_AccionsDisponibles
PROCEDURE prc_ins_tfAccDisponibles
	;
--procediment que inserta un registre en la taula de fets TF_AccionsDisponibles
PROCEDURE prc_ins_tfAccDisponibles (
	pIdAccio NUMBER
	);

-- procediment que inserta un registre en la taula de fets TF_AccioMesNegociada
PROCEDURE prc_ins_tfAccMesNegociada 
	;

-- procediment que inserta un registre en la taula de fets TF_carteraValors
PROCEDURE prc_ins_tfCarteraValors 
	;
-- procediment que inserta un registre en la taula de fets TF_Beneficis
PROCEDURE prc_ins_tfBeneficis 
	;

--Tipus RECORD i TABLE que permeten que les funcions de consultes de ranquings i agrupats retornin taules
TYPE rec_Temps IS RECORD (
		idTemps number (8),
		elAny NUMBER(4),
		elMes NUMBER(2),
		elDia NUMBER(2)
);


end pkg_dwh;
/

-- cos del paquet
CREATE OR REPLACE PACKAGE BODY pkg_dwh is

/*
* definicio de la funció d'inserció d'un registe en la dimensió TD_temps
* retorna el identificador del dia
*
*/
function fun_insDim_tdTemps (
	pData DATE  
	) 
	RETURN NUMBER
	IS
-- declaració de variables locals del procediment
	prc_nom varchar2(25 CHAR):='FUN_INSERCIO_DIM_TEMPS';
	prc_paramIN varchar(100 CHAR) := null;
	sError varchar(512);
	pidData rec_Temps;

BEGIN

-->>recull dels paràmetres d'entrada a un string
	prc_paramIN := pData;
--<<fi del recull dels paràmetres d'entrada

--pData.idTemps := to_number(to_char(pData, 'YYYYMMDD'));
	pidData.idTemps := EXTRACT(YEAR FROM pData)*10000 
			+ EXTRACT(MONTH FROM pData)*100 +  EXTRACT(DAY FROM pData) ;
	pidData.elAny := EXTRACT(YEAR FROM  pData);
	pidData.elMes := EXTRACT(MONTH FROM pData);
	pidData.elDia := EXTRACT(DAY FROM pData);

	/*DBMS_OUTPUT.PUT_LINE('el id ' || pidData.idTemps);
	DBMS_OUTPUT.PUT_LINE('el any ' || pidData.elAny);
	DBMS_OUTPUT.PUT_LINE('el mes ' || pidData.elMes);
	DBMS_OUTPUT.PUT_LINE('el dia ' || pidData.elDia);*/

-- inserció a la taula TD_Temps del registre de temps
	INSERT INTO TD_Temps
		VALUES pidData;


--registra a la taula Log l'execució del procediment
	pkg_control.prc_logProcediment(prc_nom, prc_paramIN, 'FI OK');
--s'informa del paràmetre de sortida
	
	RETURN pidData.idTemps;

	EXCEPTION
		WHEN DUP_VAL_ON_INDEX THEN --capturem l'excepció d'unicitat que indica que aquell registre de data ja existeix,
--per tant, no cal insertar-la de nou
		
--registra a la taula Log l'execució del procediment
		pkg_control.prc_logProcediment(prc_nom, prc_paramIN, 'FI OK');
--s'informa del paràmetre de sortida
		
		RETURN pidData.idTemps;

		WHEN others THEN
			sError := pkg_control.fun_error(1)||substr(SQLERRM, 1, 500);
--registra a la taula Log l'execució del procediment
			pkg_control.prc_logProcediment(prc_nom, prc_paramIN, sError);
-- s'informa del paràmetre de sortida
			RETURN NULL;

END fun_insDim_tdTemps;


/*
* definicio del procediment d'inserció d'un registe en la dimensió TD_Accio
*
*/
PROCEDURE prc_insDim_tdAccio (
	pIdAccio NUMBER
	) IS
-- declaració de variables locals del procediment
	prc_nom varchar2(25 CHAR):='PRC_INSERCIO_DIM_ACCIO';
	prc_paramIN varchar(100 CHAR) := null;
	sError varchar(512);
	reg_accio TD_accions%rowtype;

BEGIN

-->>recull dels paràmetres d'entrada a un string
	prc_paramIN := pIdAccio;
--<<fi del recull dels paràmetres d'entrada

-- obtenim la informació de l'acció
	SELECT idAccio, Ticker, nomAccio
		INTO reg_accio
		FROM accions
		WHERE idAccio = pIdAccio;

-- inserció a la taula TD_Accio del registre de l'acció
	INSERT INTO TD_Accions
		VALUES reg_accio;


--registra a la taula Log l'execució del procediment
	pkg_control.prc_logProcediment(prc_nom, prc_paramIN, 'FI OK');
--s'informa del paràmetre de sortida
	

	EXCEPTION
		WHEN DUP_VAL_ON_INDEX THEN --capturem l'excepció d'unicitat que indica que aquell registre de accio ja existeix,
--per tant, no cal insertar-la de nou
		
--registra a la taula Log l'execució del procediment
		pkg_control.prc_logProcediment(prc_nom, prc_paramIN, 'FI OK');

		WHEN NO_DATA_FOUND THEN
			sError := pkg_control.fun_error(2)||substr(SQLERRM, 1, 500);
--registra a la taula Log l'execució del procediment
			pkg_control.prc_logProcediment(prc_nom, prc_paramIN, sError);

		WHEN others THEN
			sError :=  pkg_control.fun_error(1)||substr(SQLERRM, 1, 500);
--registra a la taula Log l'execució del procediment
			pkg_control.prc_logProcediment(prc_nom, prc_paramIN, sError);

			
END prc_insDim_tdAccio;


/*
* definicio del procediment d'inserció d'un registe en la dimensió TD_Client
*
*/
PROCEDURE prc_insDim_tdClient (
	pIdClient NUMBER
	) IS
-- declaració de variables locals del procediment
	prc_nom varchar2(25 CHAR):='PRC_INSERCIO_DIM_CLIENT';
	prc_paramIN varchar(100 CHAR) := null;
	sError varchar(512);
	reg_client TD_Clients%rowtype;	

BEGIN

-->>recull dels paràmetres d'entrada a un string
	prc_paramIN := pIdClient;
--<<fi del recull dels paràmetres d'entrada

-- obtenim la informació del client
	SELECT idClient, nomClient
		INTO reg_client
		FROM clients
		WHERE idClient = pIdClient;

-- inserció a la taula TD_Clients del registre del client
	INSERT INTO TD_Clients
		VALUES reg_client;


--registra a la taula Log l'execució del procediment
	pkg_control.prc_logProcediment(prc_nom, prc_paramIN, 'FI OK');
--s'informa del paràmetre de sortida
	

	EXCEPTION
		WHEN DUP_VAL_ON_INDEX THEN --capturem l'excepció d'unicitat que indica que aquell registre de accio ja existeix,
--per tant, no cal insertar-la de nou
		
--registra a la taula Log l'execució del procediment
		pkg_control.prc_logProcediment(prc_nom, prc_paramIN, 'FI OK');
--s'informa del paràmetre de sortida
		
		WHEN NO_DATA_FOUND THEN
			sError := pkg_control.fun_error(2)||substr(SQLERRM, 1, 500);
--registra a la taula Log l'execució del procediment
			pkg_control.prc_logProcediment(prc_nom, prc_paramIN, sError);

		WHEN others THEN
			sError :=  pkg_control.fun_error(1)||substr(SQLERRM, 1, 500);
--registra a la taula Log l'execució del procediment
			pkg_control.prc_logProcediment(prc_nom, prc_paramIN, sError);
			
END prc_insDim_tdClient;


/*
* definicio del procediment d'inserció d'un registre en la taula de fets TF_AccionsDisponibles
*
*/
PROCEDURE prc_ins_tfAccDisponibles 
	 IS
-- declaració de variables locals del procediment
	prc_nom varchar2(40 CHAR):='PRC_INSERCIO_TF_ACCIONSDISP';
	prc_paramIN varchar(100 CHAR) := null;
	sError varchar(512);
	var_accio ordres.accio%type;
	var_quantitat NUMBER(8);
	var_numClients NUMBER(4);
	var_dia NUMBER(8);
	
	CURSOR cursorQtatAccions
		IS
			select accio, SUM(quantitat) as qtat
			from ordres
			where estat = 'PD' and tipus ='VENDA'
			group by accio
			order by accio;

	CURSOR cursorNumClients(pIdAccio NUMBER)
		IS
			select count (*) as ncli
			from cartera
			where accio = pIdAccio;

BEGIN

-->>recull dels paràmetres d'entrada a un string
	prc_paramIN := NULL;
--<<fi del recull dels paràmetres d'entrada

-- obtenim el identidficador del dia en curs
	var_dia := pkg_dwh.fun_insDim_tdTemps(sysdate);
 	
-- obtenim la llista amb les accions que estan a la venda i el nombre total
	OPEN cursorQtatAccions;
	LOOP
        FETCH cursorQtatAccions INTO var_accio, var_quantitat;
        EXIT WHEN cursorQtatAccions%NOTFOUND;
       
-- per a cada acció de la llista s'obté el nombre de clients que la tenen en la cartera
		OPEN cursorNumClients(var_accio);
		FETCH cursorNumClients into var_numClients;
			EXIT WHEN cursorNumClients%NOTFOUND;
			BEGIN
				INSERT INTO TF_AccionsDisponibles (idtemps, idaccio, numAccionsEnVenda, numClients)
					VALUES (var_dia, var_accio, var_quantitat, var_numClients);
			EXCEPTION
				WHEN DUP_VAL_ON_INDEX THEN
					UPDATE TF_AccionsDisponibles
						SET numAccionsEnVenda = var_quantitat, 
							numClients = var_numClients
					WHERE idtemps = var_dia AND idaccio = var_accio;
			END;

		CLOSE cursorNumClients;

	END LOOP;
	CLOSE cursorQtatAccions;

--registra a la taula Log l'execució del procediment
	pkg_control.prc_logProcediment(prc_nom, prc_paramIN, 'FI OK');
--s'informa del paràmetre de sortida

	EXCEPTION
		WHEN NO_DATA_FOUND THEN
			sError := pkg_control.fun_error(2)||substr(SQLERRM, 1, 500);
--registra a la taula Log l'execució del procediment
			pkg_control.prc_logProcediment(prc_nom, prc_paramIN, sError);

		WHEN others THEN
			sError :=  pkg_control.fun_error(1)||substr(SQLERRM, 1, 500);
--registra a la taula Log l'execució del procediment
			pkg_control.prc_logProcediment(prc_nom, prc_paramIN, sError);
			
END prc_ins_tfAccDisponibles;

/*
* definicio del procediment d'inserció d'un registre en la taula de fets TF_AccionsDisponibles
* sobrecàrrega de mètode. Se li passa el Id de l'acció que es posa a la venda o s'executa
*/
PROCEDURE prc_ins_tfAccDisponibles (
	pIdAccio NUMBER
	) IS
-- declaració de variables locals del procediment
	prc_nom varchar2(40 CHAR):='PRC_INSERCIO_TF_ACCIONSDISP';
	prc_paramIN varchar(100 CHAR) := null;
	sError varchar(512);
	
	var_quantitat NUMBER(8) := 0;
	var_numClients NUMBER(4):= 0;
	var_dia NUMBER(8);
	

BEGIN

-->>recull dels paràmetres d'entrada a un string
	prc_paramIN := NULL;
--<<fi del recull dels paràmetres d'entrada

-- obtenim el identidficador del dia en curs
	var_dia := pkg_dwh.fun_insDim_tdTemps(sysdate);
 	
-- obtenim la quantitat de les accions que estan a la venda del mateix tipus i el nombre  de clients que en tenen
	
	select SUM(quantitat)
		into var_quantitat
		from ordres
		where estat = 'PD' and tipus ='VENDA' and accio = pIdAccio
		;

	select count (*)
		into var_numClients
		from cartera
		where accio = pIdAccio
		;

	
	var_quantitat := NVL (var_quantitat, 0);
	var_numClients := NVL (var_numClients, 0);

	BEGIN
		INSERT INTO TF_AccionsDisponibles (idtemps, idaccio, numAccionsEnVenda, numClients)
			VALUES (var_dia, pIdAccio, var_quantitat, var_numClients);
	EXCEPTION
		WHEN DUP_VAL_ON_INDEX THEN
			UPDATE TF_AccionsDisponibles
				SET numAccionsEnVenda = var_quantitat, 
					numClients = var_numClients
			WHERE idtemps = var_dia AND idaccio = pIdAccio;
	END;


--registra a la taula Log l'execució del procediment
	pkg_control.prc_logProcediment(prc_nom, prc_paramIN, 'FI OK');
--s'informa del paràmetre de sortida

	EXCEPTION
		WHEN NO_DATA_FOUND THEN
			sError := pkg_control.fun_error(2)||substr(SQLERRM, 1, 500);
--registra a la taula Log l'execució del procediment
			pkg_control.prc_logProcediment(prc_nom, prc_paramIN, sError);

		WHEN others THEN
			sError :=  pkg_control.fun_error(1)||substr(SQLERRM, 1, 500);
--registra a la taula Log l'execució del procediment
			pkg_control.prc_logProcediment(prc_nom, prc_paramIN, sError);
			
END prc_ins_tfAccDisponibles;




		
/*
* definicio del procediment d'inserció d'un registe en la taula de fets TF_accioMesNegociada
* Es refereix a l'acció que ha participat en més operacions de compra/venda
*/
PROCEDURE prc_ins_tfAccMesNegociada 
	 IS
-- declaració de variables locals del procediment
	prc_nom varchar2(40 CHAR):='PRC_INSERCIO_TF_ACCIOMESNEGOCIADA';
	prc_paramIN varchar(100 CHAR) := null;
	sError varchar(512);
	var_dia NUMBER(8);

	CURSOR cursorNumOperacions
		IS
			SELECT C1.*
				FROM (
					SELECT O.ACCIO, COUNT(*) AS QTAT
					FROM  ORDRES O
					WHERE O.ESTAT = 'EX'
					GROUP BY O.ACCIO ) C1
				JOIN (
					SELECT MAX (C2.QTAT) AS TOPVALOR
					FROM (SELECT O2.ACCIO, COUNT(*) AS QTAT
						FROM ORDRES O2
						WHERE O2.ESTAT = 'EX'
						GROUP BY O2.ACCIO) C2)C3
				ON C1.QTAT = C3.TOPVALOR;

BEGIN

-->>recull dels paràmetres d'entrada a un string
	prc_paramIN := NULL;
--<<fi del recull dels paràmetres d'entrada

-- obtenim el identificador del dia en curs
	var_dia := pkg_dwh.fun_insDim_tdTemps(sysdate);
 
-- obtenim el grup d'accions més negociades, en el cas que hi hagi més d'una acció amb el valor màxim de 
-- operacions, retornarà un grup de registres, i s'inserta en la TF_accioMesNegociada
	FOR reg_numOp IN cursorNumOperacions
	LOOP
		BEGIN
			INSERT INTO TF_accioMesNegociada (idTemps, idAccio, numOperacions)
				VALUES (var_dia, reg_numOp.accio, reg_numOp.qtat );
			
		EXCEPTION
				WHEN DUP_VAL_ON_INDEX THEN
					UPDATE TF_accioMesNegociada
						SET numOperacions = reg_numOp.qtat
						WHERE idtemps = var_dia AND idaccio = reg_numOp.accio;
			END;

	END LOOP;

--registra a la taula Log l'execució del procediment
	pkg_control.prc_logProcediment(prc_nom, prc_paramIN, 'FI OK');
--s'informa del paràmetre de sortida

	EXCEPTION
		WHEN NO_DATA_FOUND THEN
			sError := pkg_control.fun_error(2)||substr(SQLERRM, 1, 500);
--registra a la taula Log l'execució del procediment
			pkg_control.prc_logProcediment(prc_nom, prc_paramIN, sError);

		WHEN others THEN
			sError :=  pkg_control.fun_error(1)||substr(SQLERRM, 1, 500);
--registra a la taula Log l'execució del procediment
			pkg_control.prc_logProcediment(prc_nom, prc_paramIN, sError);	

END prc_ins_tfAccMesNegociada;


/*
* definicio del procediment d'inserció d'un registe en la taula de fets TF_carteraValors
* Es refereix al client que té una cartera d'accions amb un valor més elevat, i no es refereix
* al client amb el major nombre d'accions.
*/
PROCEDURE prc_ins_tfCarteraValors 
	 IS
-- declaració de variables locals del procediment
	prc_nom varchar2(40 CHAR):='PRC_INSERCIO_TF_CARTERAVALORS';
	prc_paramIN varchar(100 CHAR) := null;
	sError varchar(512);
	var_dia NUMBER(8);

	CURSOR cursorValorMaxCartera
		IS
			SELECT C1.*
				FROM (
					SELECT CA.CLIENT, SUM(QUANTITAT) AS QTAT, SUM(VALORTOTAL) AS VALOR
					FROM CARTERA CA
					GROUP BY CA.CLIENT
				) C1
				JOIN (
					SELECT MAX(C2.VALOR) AS MAXVALOR
					FROM (SELECT CA2.CLIENT, SUM(QUANTITAT) AS QTAT, SUM(VALORTOTAL) AS VALOR
					      FROM CARTERA CA2
					      GROUP BY CA2.CLIENT) C2)C3
				ON C1.VALOR=C3.MAXVALOR;

	CURSOR cursorQtatMaxCartera
		IS
			SELECT C1.*
				FROM (
					SELECT CA.CLIENT, SUM(QUANTITAT) AS QTAT, SUM(VALORTOTAL) AS VALOR
					FROM CARTERA CA
					GROUP BY CA.CLIENT
				) C1
				JOIN (
					SELECT MAX(C2.QTAT) AS MAXQTAT
					FROM (SELECT CA2.CLIENT, SUM(QUANTITAT) AS QTAT, SUM(VALORTOTAL) AS VALOR
					      FROM CARTERA CA2
					      GROUP BY CA2.CLIENT) C2)C3
				ON C1.QTAT=C3.MAXQTAT;

BEGIN

-->>recull dels paràmetres d'entrada a un string
	prc_paramIN := NULL;
--<<fi del recull dels paràmetres d'entrada

-- obtenim el identificador del dia en curs
	var_dia := pkg_dwh.fun_insDim_tdTemps(sysdate);

-- obtenim el grup clients que tenen una cartera més gran, es pot donar el cas de diversos clients tinguin el mateix
-- valor max, i s'inserta en la TF_carteraValors
	FOR reg_valCartera IN cursorValorMaxCartera
	LOOP
		BEGIN
			INSERT INTO TF_carteraValors (idTemps, idClient, numaccions, valorcartera)
				VALUES (var_dia, reg_valCartera.client, reg_valCartera.qtat, reg_valCartera.valor);
		EXCEPTION
				WHEN DUP_VAL_ON_INDEX THEN
					UPDATE TF_carteraValors
						SET numaccions = reg_valCartera.qtat, valorcartera = reg_valCartera.valor
						WHERE idtemps = var_dia AND idClient = reg_valCartera.client;
		END;

	END LOOP;

--registra a la taula Log l'execució del procediment
	pkg_control.prc_logProcediment(prc_nom, prc_paramIN, 'FI OK');
--s'informa del paràmetre de sortida

	EXCEPTION
		WHEN NO_DATA_FOUND THEN
			sError := pkg_control.fun_error(2)||substr(SQLERRM, 1, 500);
--registra a la taula Log l'execució del procediment
			pkg_control.prc_logProcediment(prc_nom, prc_paramIN, sError);

		WHEN others THEN
			sError :=  pkg_control.fun_error(1)||substr(SQLERRM, 1, 500);
--registra a la taula Log l'execució del procediment
			pkg_control.prc_logProcediment(prc_nom, prc_paramIN, sError);


END prc_ins_tfCarteraValors;


/*
* definicio del procediment d'inserció d'un registe en la taula de fets TF_beneficis
* Es refereix al diferencial de valor entre principi de mes i la data actual ( o fi de mes)
*/
PROCEDURE prc_ins_tfBeneficis 
	 IS
-- declaració de variables locals del procediment
	prc_nom varchar2(40 CHAR):='PRC_INSERCIO_TF_BENEFICIS';
	prc_paramIN varchar(100 CHAR) := null;
	sError varchar(512);
	var_dia NUMBER(8);
	
	var_dataIni TIMESTAMP;
	var_dataFi TIMESTAMP;

	var_valor1er NUMBER(8);
	var_valor2on NUMBER(8);
	var_beneficiNet NUMBER(8);
	var_x100Benef NUMBER(5,2);

	trobat BOOLEAN := false;

	CURSOR cursorClientsCartera
		IS
			select distinct client
			from cartera; 



BEGIN
	
-->>recull dels paràmetres d'entrada a un string
	prc_paramIN := NULL;
--<<fi del recull dels paràmetres d'entrada

-- obtenim el identificador del dia en curs
	var_dia := pkg_dwh.fun_insDim_tdTemps(sysdate);
-- TRUNC(SYSDATE, 'MM'); -- obtenim el 1er dia del mes en curs

	FOR reg_clientCartera IN cursorClientsCartera
	LOOP
		select MIN(hc.datainici)
			INTO var_dataIni
		  from historiccartera hc
		  join
		  cartera ca
		    on hc.cartera = ca.idcartera
		  WHERE HC.DATAINICI BETWEEN TRUNC(SYSDATE, 'MM') AND SYSDATE
		  and ca.client = reg_clientCartera.client;

		select MAX(hc.datainici)
			INTO var_dataFi
		  from historiccartera hc
		  join
		  cartera ca
		    on hc.cartera = ca.idcartera
		  WHERE HC.DATAINICI BETWEEN TRUNC(SYSDATE, 'MM') AND SYSDATE
		  and ca.client = reg_clientCartera.client;


		IF var_dataIni IS NOT NULL 
		THEN

			select sum(hc.valorTotal)
				into var_valor1er
				from historiccartera hc join cartera ca
      			on hc.cartera = ca.idcartera
				where ca.client = reg_clientCartera.client and trunc(hc.datainici, 'dd') = trunc(var_dataIni, 'dd');

				
			IF var_dataFi IS NOT NULL and var_dataFi <> var_dataIni
			THEN

				select sum(hc.valorTotal)
					into var_valor2on
					from historiccartera hc join cartera ca
	      			on hc.cartera = ca.idcartera
					where ca.client = reg_clientCartera.client and trunc(hc.datainici, 'dd') = trunc(var_dataFi, 'dd');

			
				var_beneficiNet := var_valor2on - var_valor1er;
				var_x100Benef := ((var_beneficiNet)/var_valor1er)*100;
			
				BEGIN

					INSERT INTO TF_beneficis (idtemps, idclient, beneficinet, x100benefici)
						VALUES (var_dia, reg_clientCartera.client,var_beneficiNet, var_x100Benef);
				EXCEPTION
					WHEN DUP_VAL_ON_INDEX THEN
						UPDATE TF_beneficis
							SET beneficinet = var_beneficiNet, x100benefici = var_x100Benef
							WHERE idtemps = var_dia AND idClient = reg_clientCartera.client;

				END;	

			ELSE 

				BEGIN

					INSERT INTO TF_beneficis (idtemps, idclient, beneficinet, x100benefici)
						VALUES (var_dia, reg_clientCartera.client,var_valor1er, 100);
				EXCEPTION
					WHEN DUP_VAL_ON_INDEX THEN
						UPDATE TF_beneficis
							SET beneficinet = var_valor1er, x100benefici = 100
							WHERE idtemps = var_dia AND idClient = reg_clientCartera.client;

				END;	



			END IF;


		ELSIF var_dataFi IS NOT NULL
			THEN

			select valorTotal
				into var_valor2on
				from historiccartera
				where datainici = var_dataFi;

			BEGIN

					INSERT INTO TF_beneficis (idtemps, idclient, beneficinet, x100benefici)
						VALUES (var_dia, reg_clientCartera.client,var_valor2on, 100);
			EXCEPTION
				WHEN DUP_VAL_ON_INDEX THEN
					UPDATE TF_beneficis
						SET beneficinet = var_valor2on, x100benefici = 100
						WHERE idtemps = var_dia AND idClient = reg_clientCartera.client;

			END;

		END IF;

	
	END LOOP;



	--registra a la taula Log l'execució del procediment
	pkg_control.prc_logProcediment(prc_nom, prc_paramIN, 'FI OK');
--s'informa del paràmetre de sortida

	EXCEPTION
		WHEN NO_DATA_FOUND THEN
			sError := pkg_control.fun_error(2)||substr(SQLERRM, 1, 500);
--registra a la taula Log l'execució del procediment
			pkg_control.prc_logProcediment(prc_nom, prc_paramIN, sError);

		WHEN others THEN
			sError :=  pkg_control.fun_error(1)||substr(SQLERRM, 1, 500);
--registra a la taula Log l'execució del procediment
			pkg_control.prc_logProcediment(prc_nom, prc_paramIN, sError);	

END prc_ins_tfBeneficis;


-- fi del paquet
END pkg_dwh;
/
spool off;